/*
 * Copyright (C) 2008-2013 TrinityCore <http://www.trinitycore.org/>
 * Copyright (C) 2005-2009 MaNGOS <http://getmangos.com/>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/// \addtogroup u2w
/// @{
/// \file

#ifndef _OPCODES_H
#define _OPCODES_H

#include "Common.h"

/// List of Opcodes
enum Opcodes
{
    NUM_OPCODE_HANDLERS                               = (0x7FFF+1),
    UNKNOWN_OPCODE                                    = (0xFFFF+1),
    NULL_OPCODE                                       = 0,
    COMPRESSED_OPCODE_MASK                            = 0x8000,

    CMSG_ACCEPT_LEVEL_GRANT                           = 0x0205,
    CMSG_ACCEPT_TRADE                                 = 0x7110,
    CMSG_ACTIVATETAXI                                 = 0x6E06,
    CMSG_ACTIVATETAXIEXPRESS                          = 0x0515,
    CMSG_ADDON_REGISTERED_PREFIXES                    = 0x0954,
    CMSG_ADD_FRIEND                                   = 0x6527,
    CMSG_ADD_IGNORE                                   = 0x4726,
    CMSG_ADD_VOICE_IGNORE                             = 0x0F06,
    CMSG_ALTER_APPEARANCE                             = 0x0914,
    CMSG_AREATRIGGER                                  = 0x0937,
    CMSG_AREA_SPIRIT_HEALER_QUERY                     = 0x4907,
    CMSG_AREA_SPIRIT_HEALER_QUEUE                     = 0x4815,
    CMSG_ARENA_TEAM_ACCEPT                            = 0x2A25,
    CMSG_ARENA_TEAM_CREATE                            = 0x04A1,
    CMSG_ARENA_TEAM_DECLINE                           = 0x6925,
    CMSG_ARENA_TEAM_DISBAND                           = 0x6504,
    CMSG_ARENA_TEAM_INVITE                            = 0x2F27,
    CMSG_ARENA_TEAM_LEADER                            = 0x4204,
    CMSG_ARENA_TEAM_LEAVE                             = 0x0E16,
    CMSG_ARENA_TEAM_QUERY                             = 0x0514,
    CMSG_ARENA_TEAM_REMOVE                            = 0x2F05,
    CMSG_ARENA_TEAM_ROSTER                            = 0x6F37,
    CMSG_ATTACKSTOP                                   = 0x4106,
    CMSG_ATTACKSWING                                  = 0x0926,
    CMSG_AUCTION_LIST_BIDDER_ITEMS                    = 0x6937,
    CMSG_AUCTION_LIST_ITEMS                           = 0x0324,
    CMSG_AUCTION_LIST_OWNER_ITEMS                     = 0x0206,
    CMSG_AUCTION_LIST_PENDING_SALES                   = 0x2C17,
    CMSG_AUCTION_PLACE_BID                            = 0x2306,
    CMSG_AUCTION_REMOVE_ITEM                          = 0x6426,
    CMSG_AUCTION_SELL_ITEM                            = 0x4A06,
    CMSG_AUTH_SESSION                                 = 0x0449,
    CMSG_AUTOBANK_ITEM                                = 0x2537,
    CMSG_AUTOEQUIP_GROUND_ITEM                        = 0x0000,
    CMSG_AUTOEQUIP_ITEM                               = 0x4304,
    CMSG_AUTOEQUIP_ITEM_SLOT                          = 0x4A17,
    CMSG_AUTOSTORE_BAG_ITEM                           = 0x0236,
    CMSG_AUTOSTORE_BANK_ITEM                          = 0x0607,
    CMSG_AUTOSTORE_GROUND_ITEM                        = 0x0000,
    CMSG_AUTOSTORE_LOOT_ITEM                          = 0x0E34,
    CMSG_AUTO_DECLINE_GUILD_INVITES                   = 0x2034,
    CMSG_BANKER_ACTIVATE                              = 0x0005,
    CMSG_BATTLEFIELD_JOIN                             = 0x0000,
    CMSG_BATTLEFIELD_LEAVE                            = 0x3018,
    CMSG_BATTLEFIELD_LIST                             = 0x3814,
    CMSG_BATTLEFIELD_MGR_ENTRY_INVITE_RESPONSE        = 0x05A3,
    CMSG_BATTLEFIELD_MGR_EXIT_REQUEST                 = 0x2490,
    CMSG_BATTLEFIELD_MGR_QUEUE_INVITE_RESPONSE        = 0x0413,
    CMSG_BATTLEFIELD_MGR_QUEUE_REQUEST                = 0x710C,
    CMSG_BATTLEFIELD_PORT                             = 0x711A,
    CMSG_BATTLEFIELD_REQUEST_SCORE_DATA               = 0x0000,
    CMSG_BATTLEFIELD_STATUS                           = 0x2500,
    CMSG_BATTLEGROUND_PLAYER_POSITIONS                = 0x3902,
    CMSG_BATTLEMASTER_JOIN                            = 0x7902,
    CMSG_BATTLEMASTER_JOIN_ARENA                      = 0x701C,
    CMSG_BATTLEMASTER_JOIN_RATED                      = 0x3B18,
    CMSG_BEGIN_TRADE                                  = 0x721E,
    CMSG_BINDER_ACTIVATE                              = 0x4006,
    CMSG_BOT_DETECTED2                                = 0x0000,
    CMSG_BUG                                          = 0x4035,
    CMSG_BUSY_TRADE                                   = 0x331C,
    CMSG_BUYBACK_ITEM                                 = 0x6C17,
    CMSG_BUY_BANK_SLOT                                = 0x0425,
    CMSG_BUY_ITEM                                     = 0x0736,
    CMSG_CALENDAR_ADD_EVENT                           = 0x0726,
    CMSG_CALENDAR_ARENA_TEAM                          = 0x0204,
    CMSG_CALENDAR_COMPLAIN                            = 0x4C36,
    CMSG_CALENDAR_CONTEXT_EVENT_SIGNUP                = 0x0000,
    CMSG_CALENDAR_COPY_EVENT                          = 0x0207,
    CMSG_CALENDAR_EVENT_INVITE                        = 0x2435,
    CMSG_CALENDAR_EVENT_MODERATOR_STATUS              = 0x6B35,
    CMSG_CALENDAR_EVENT_REMOVE_INVITE                 = 0x4337,
    CMSG_CALENDAR_EVENT_RSVP                          = 0x0227,
    CMSG_CALENDAR_EVENT_SIGNUP                        = 0x6606,
    CMSG_CALENDAR_EVENT_STATUS                        = 0x2D24,
    CMSG_CALENDAR_GET_CALENDAR                        = 0x2814,
    CMSG_CALENDAR_GET_EVENT                           = 0x6416,
    CMSG_CALENDAR_GET_NUM_PENDING                     = 0x4D05,
    CMSG_CALENDAR_GUILD_FILTER                        = 0x4A16,
    CMSG_CALENDAR_REMOVE_EVENT                        = 0x6636,
    CMSG_CALENDAR_UPDATE_EVENT                        = 0x2114,
    CMSG_CANCEL_AURA                                  = 0x0E26,
    CMSG_CANCEL_AUTO_REPEAT_SPELL                     = 0x6C35,
    CMSG_CANCEL_CAST                                  = 0x0115,
    CMSG_CANCEL_CHANNELLING                           = 0x6C25,
    CMSG_CANCEL_GROWTH_AURA                           = 0x0000,
    CMSG_CANCEL_MOUNT_AURA                            = 0x0635,
    CMSG_CANCEL_QUEUED_SPELL                          = 0x7B1C,
    CMSG_CANCEL_TEMP_ENCHANTMENT                      = 0x6C37,
    CMSG_CANCEL_TRADE                                 = 0x731E,
    CMSG_CAST_SPELL                                   = 0x4C07,
    CMSG_CHANGEPLAYER_DIFFICULTY                      = 0x6107,
    CMSG_CHANGE_SEATS_ON_CONTROLLED_VEHICLE           = 0x7310,
    CMSG_CHANNEL_ANNOUNCEMENTS                        = 0x1146,
    CMSG_CHANNEL_BAN                                  = 0x3D56,
    CMSG_CHANNEL_DISPLAY_LIST                         = 0x2144,
    CMSG_CHANNEL_INVITE                               = 0x0144,
    CMSG_CHANNEL_KICK                                 = 0x3156,
    CMSG_CHANNEL_LIST                                 = 0x1556,
    CMSG_CHANNEL_MODERATE                             = 0x2944,
    CMSG_CHANNEL_MODERATOR                            = 0x0146,
    CMSG_CHANNEL_MUTE                                 = 0x2554,
    CMSG_CHANNEL_OWNER                                = 0x3D44,
    CMSG_CHANNEL_PASSWORD                             = 0x2556,
    CMSG_CHANNEL_ROSTER_INFO                          = 0x3546,
    CMSG_CHANNEL_SET_OWNER                            = 0x3556,
    CMSG_CHANNEL_SILENCE_ALL                          = 0x2154,
    CMSG_CHANNEL_SILENCE_VOICE                        = 0x2D54,
    CMSG_CHANNEL_UNBAN                                = 0x2D46,
    CMSG_CHANNEL_UNMODERATOR                          = 0x1954,
    CMSG_CHANNEL_UNMUTE                               = 0x3554,
    CMSG_CHANNEL_UNSILENCE_ALL                        = 0x2546,
    CMSG_CHANNEL_UNSILENCE_VOICE                      = 0x3146,
    CMSG_CHANNEL_VOICE_OFF                            = 0x3144,
    CMSG_CHANNEL_VOICE_ON                             = 0x1144,
    CMSG_CHAR_CREATE                                  = 0x4A36,
    CMSG_CHAR_CUSTOMIZE                               = 0x2C34,
    CMSG_CHAR_DELETE                                  = 0x6425,
    CMSG_CHAR_ENUM                                    = 0x0502,
    CMSG_CHAR_FACTION_CHANGE                          = 0x2735,
    CMSG_COMPLETED_ARTIFACTS                          = 0x0A13,
    CMSG_CHAR_RACE_CHANGE                             = 0x0D24,
    CMSG_CHAR_RENAME                                  = 0x2327,
    CMSG_CHAT_FILTERED                                = 0x0946,
    CMSG_CHAT_IGNORED                                 = 0x0D54,
    CMSG_CLEAR_CHANNEL_WATCH                          = 0x2604,
    CMSG_CLEAR_RAID_MARKER                            = 0x7300,
    CMSG_CLEAR_TRADE_ITEM                             = 0x7018,
    CMSG_COMMENTATOR_ENABLE                           = 0x0B07,
    CMSG_COMMENTATOR_ENTER_INSTANCE                   = 0x4105,
    CMSG_COMMENTATOR_EXIT_INSTANCE                    = 0x6136,
    CMSG_COMMENTATOR_GET_MAP_INFO                     = 0x0026,
    CMSG_COMMENTATOR_GET_PARTY_INFO                   = 0x2412,
    CMSG_COMMENTATOR_GET_PLAYER_INFO                  = 0x0D14,
    CMSG_COMMENTATOR_INSTANCE_COMMAND                 = 0x0917,
    CMSG_COMMENTATOR_SKIRMISH_QUEUE_COMMAND           = 0x0025,
    CMSG_COMMENTATOR_START_WARGAME                    = 0x25A0,
    CMSG_COMPLAIN                                     = 0x0427,
    CMSG_COMPLETE_CINEMATIC                           = 0x2116,
    CMSG_COMPLETE_MOVIE                               = 0x4136,
    CMSG_CONNECT_TO_FAILED                            = 0x2533,
    CMSG_CONTACT_LIST                                 = 0x4534,
    CMSG_CORPSE_MAP_POSITION_QUERY                    = 0x6205,
    CMSG_CREATURE_QUERY                               = 0x2706,
    CMSG_DANCE_QUERY                                  = 0x4E07,
    CMSG_DECLINE_CHANNEL_INVITE                       = 0x0000,
    CMSG_DELETEEQUIPMENT_SET                          = 0x4D07,
    CMSG_DEL_FRIEND                                   = 0x6A15,
    CMSG_DEL_IGNORE                                   = 0x6D26,
    CMSG_DEL_VOICE_IGNORE                             = 0x0024,
    CMSG_DESTROY_ITEM                                 = 0x4A27,
    CMSG_DISMISS_CONTROLLED_VEHICLE                   = 0x3218,
    CMSG_DISMISS_CRITTER                              = 0x4227,
    CMSG_DUEL_ACCEPTED                                = 0x2136,
    CMSG_DUEL_CANCELLED                               = 0x6624,
    CMSG_LFG_LOCK_INFO_REQUEST                      = 0x0412,
    CMSG_EJECT_PASSENGER                              = 0x6927,
    CMSG_EMOTE                                        = 0x4C26,
    CMSG_ENABLETAXI                                   = 0x0C16,
    CMSG_ENABLE_NAGLE                                 = 0x4449,
    CMSG_EQUIPMENT_SET_DELETE                         = 0x0000,
    CMSG_EQUIPMENT_SET_SAVE                           = 0x4F27,
    CMSG_EQUIPMENT_SET_USE                            = 0x0417,
    CMSG_FAR_SIGHT                                    = 0x4835,
    CMSG_FORCE_MOVE_ROOT_ACK                          = 0x701E,
    CMSG_FORCE_MOVE_UNROOT_ACK                        = 0x7808,
    CMSG_GAMEOBJECT_QUERY                             = 0x4017,
    CMSG_GAMEOBJ_REPORT_USE                           = 0x4827,
    CMSG_GAMEOBJ_USE                                  = 0x4E17,
    CMSG_GAMESPEED_SET                                = 0x0000,
    CMSG_GAMETIME_SET                                 = 0x0000,
    CMSG_GETDEATHBINDZONE                             = 0x0000,
    CMSG_GET_MAIL_LIST                                = 0x4D37,
    CMSG_GET_MIRRORIMAGE_DATA                         = 0x0C25,
    CMSG_GHOST                                        = 0x0000,
    CMSG_GMRESPONSE_RESOLVE                           = 0x6506,
    CMSG_GMSURVEY_SUBMIT                              = 0x2724,
    CMSG_GMTICKET_CREATE                              = 0x0137,
    CMSG_GMTICKET_DELETETICKET                        = 0x6B14,
    CMSG_GMTICKET_GETTICKET                           = 0x0326,
    CMSG_GMTICKET_SYSTEMSTATUS                        = 0x4205,
    CMSG_GMTICKET_UPDATETEXT                          = 0x0636,
    CMSG_GM_INVIS                                     = 0x0000,
    CMSG_GM_NUKE                                      = 0x0000,
    CMSG_GM_REPORT_LAG                                = 0x6726,
    CMSG_GM_SET_SECURITY_GROUP                        = 0x0000,
    CMSG_GOSSIP_HELLO                                 = 0x4525,
    CMSG_GOSSIP_SELECT_OPTION                         = 0x0216,
    CMSG_GRANT_LEVEL                                  = 0x6D16,
    CMSG_GROUP_ASSISTANT_LEADER                       = 0x6025,
    CMSG_GROUP_CANCEL                                 = 0x0000,
    CMSG_GROUP_CHANGE_SUB_GROUP                       = 0x4124,
    CMSG_GROUP_DISBAND                                = 0x2804,
    CMSG_GROUP_INVITE                                 = 0x0513,
    CMSG_GROUP_INVITE_RESPONSE                        = 0x0410,
    CMSG_GROUP_RAID_CONVERT                           = 0x6E27,
    CMSG_GROUP_REQUEST_JOIN_UPDATES                   = 0x2583,
    CMSG_GROUP_SET_LEADER                             = 0x4C17,
    CMSG_GROUP_SET_ROLES                              = 0x25B1,
    CMSG_GROUP_SWAP_SUB_GROUP                         = 0x0034,
    CMSG_GROUP_UNINVITE                               = 0x0000,
    CMSG_GROUP_UNINVITE_GUID                          = 0x2E07,
    CMSG_GUILD_ACCEPT                                 = 0x2531,
    CMSG_GUILD_ACHIEVEMENT_MEMBERS                    = 0x3025,
    CMSG_GUILD_ACHIEVEMENT_PROGRESS_QUERY             = 0x3235,
    CMSG_GUILD_ADD_RANK                               = 0x3030,
    CMSG_GUILD_ASSIGN_MEMBER_RANK                     = 0x3032,
    CMSG_GUILD_BANKER_ACTIVATE                        = 0x2E37,
    CMSG_GUILD_BANK_BUY_TAB                           = 0x0C37,
    CMSG_GUILD_BANK_DEPOSIT_MONEY                     = 0x0707,
    CMSG_GUILD_BANK_LOG_QUERY                         = 0x3224,
    CMSG_GUILD_BANK_MONEY_WITHDRAWN_QUERY             = 0x1225,
    CMSG_GUILD_BANK_NOTE                              = 0x0000,
    CMSG_GUILD_BANK_QUERY_TAB                         = 0x2E35,
    CMSG_GUILD_BANK_QUERY_TEXT                        = 0x3220,
    CMSG_GUILD_BANK_SWAP_ITEMS                        = 0x2315,
    CMSG_GUILD_BANK_UPDATE_TAB                        = 0x0106,
    CMSG_GUILD_BANK_WITHDRAW_MONEY                    = 0x0037,
    CMSG_GUILD_CHANGE_NAME_REQUEST                    = 0x1232,
    CMSG_GUILD_DECLINE                                = 0x3231,
    CMSG_GUILD_DEL_RANK                               = 0x3234,
    CMSG_GUILD_DEMOTE                                 = 0x1020,
    CMSG_GUILD_DISBAND                                = 0x3226,
    CMSG_GUILD_EVENT_LOG_QUERY                        = 0x1220,
    CMSG_GUILD_INFO                                   = 0x0000,
    CMSG_GUILD_INFO_TEXT                              = 0x3227,
    CMSG_GUILD_INVITE                                 = 0x24B0,
    CMSG_GUILD_LEAVE                                  = 0x1021,
    CMSG_GUILD_MEMBER_SEND_SOR_REQUEST                = 0x3225,
    CMSG_GUILD_MOTD                                   = 0x1035,
    CMSG_GUILD_NEWS_UPDATE_STICKY                     = 0x3223,
    CMSG_GUILD_PERMISSIONS                            = 0x3022,
    CMSG_GUILD_PROMOTE                                = 0x1030,
    CMSG_GUILD_QUERY                                  = 0x4426,
    CMSG_GUILD_QUERY_NEWS                             = 0x3020,
    CMSG_GUILD_QUERY_RANKS                            = 0x1026,
    CMSG_GUILD_REMOVE                                 = 0x1231,
    CMSG_GUILD_REPLACE_GUILD_MASTER                   = 0x1034,
    CMSG_GUILD_REQUEST_CHALLENGE_UPDATE               = 0x1224,
    CMSG_GUILD_REQUEST_MAX_DAILY_XP                   = 0x3232,
    CMSG_GUILD_REQUEST_PARTY_STATE                    = 0x3900,
    CMSG_GUILD_ROSTER                                 = 0x1226,
    CMSG_GUILD_SET_ACHIEVEMENT_TRACKING               = 0x1027,
    CMSG_GUILD_SET_GUILD_MASTER                       = 0x3034,
    CMSG_GUILD_SET_NOTE                               = 0x1233,
    CMSG_GUILD_SET_RANK_PERMISSIONS                   = 0x1024,
    CMSG_GUILD_SWITCH_RANK                            = 0x1221,
    CMSG_HEARTH_AND_RESURRECT                         = 0x4B34,
    CMSG_IGNORE_TRADE                                 = 0x7112,
    CMSG_INITIATE_TRADE                               = 0x7916,
    CMSG_INSPECT                                      = 0x0927,
    CMSG_INSPECT_HONOR_STATS                          = 0x791E,
    CMSG_INSTANCE_LOCK_WARNING_RESPONSE               = 0x6234,
    CMSG_ITEM_REFUND                                  = 0x6134,
    CMSG_ITEM_REFUND_INFO                             = 0x2206,
    CMSG_ITEM_TEXT_QUERY                              = 0x2406,
    CMSG_JOIN_CHANNEL                                 = 0x0156,
    CMSG_KEEP_ALIVE                                   = 0x0015,
    CMSG_LEARN_PREVIEW_TALENTS                        = 0x2415,
    CMSG_LEARN_PREVIEW_TALENTS_PET                    = 0x6E24,
    CMSG_LEARN_TALENT                                 = 0x0306,
    CMSG_LEAVE_CHANNEL                                = 0x2D56,
    CMSG_LFG_GET_PLAYER_INFO                          = 0x0000,
    CMSG_LFG_GET_STATUS                               = 0x2581,
    CMSG_LFG_JOIN                                     = 0x2430,
    CMSG_LFG_LEAVE                                    = 0x2433,
    CMSG_LFG_LFR_JOIN                                 = 0x0531,
    CMSG_LFG_LFR_LEAVE                                = 0x0500,
    //CMSG_LFG_LOCK_INFO_REQUEST                        = 0x0412,
    CMSG_LFG_PROPOSAL_RESULT                          = 0x0403,
    CMSG_LFG_SET_BOOT_VOTE                            = 0x04B3,
    CMSG_LFG_SET_COMMENT                              = 0x0530,
    CMSG_LFG_SET_ROLES                                = 0x0480,
    CMSG_LFG_TELEPORT                                 = 0x2482,
    CMSG_LF_GUILD_ADD_RECRUIT                         = 0x4448,
    CMSG_LF_GUILD_BROWSE                              = 0x0548,
    CMSG_LF_GUILD_DECLINE_RECRUIT                     = 0x1031,
    CMSG_LF_GUILD_GET_APPLICATIONS                    = 0x1230,
    CMSG_LF_GUILD_GET_RECRUITS                        = 0x3230,
    CMSG_LF_GUILD_POST_REQUEST                        = 0x3237,
    CMSG_LF_GUILD_REMOVE_RECRUIT                      = 0x3027,
    CMSG_LF_GUILD_SET_GUILD_POST                      = 0x0448,
    CMSG_LIST_INVENTORY                               = 0x2806,
    CMSG_LOAD_SCREEN                                  = 0x2422,
    CMSG_LOGOUT_CANCEL                                = 0x2324,
    CMSG_LOGOUT_REQUEST                               = 0x0A25,
    CMSG_LOG_DISCONNECT                               = 0x446D,
    CMSG_LOOT                                         = 0x0127,
    CMSG_LOOT_CURRENCY                                = 0x781C,
    CMSG_LOOT_MASTER_GIVE                             = 0x4F35,
    CMSG_LOOT_METHOD                                  = 0x2F24,
    CMSG_LOOT_MONEY                                   = 0x6227,
    CMSG_LOOT_RELEASE                                 = 0x2007,
    CMSG_LOOT_ROLL                                    = 0x6934,
    CMSG_MAIL_CREATE_TEXT_ITEM                        = 0x0B14,
    CMSG_MAIL_DELETE                                  = 0x6104,
    CMSG_MAIL_MARK_AS_READ                            = 0x0C07,
    CMSG_MAIL_RETURN_TO_SENDER                        = 0x0816,
    CMSG_MAIL_TAKE_ITEM                               = 0x2B06,
    CMSG_MAIL_TAKE_MONEY                              = 0x4034,
    CMSG_MEETINGSTONE_INFO                            = 0x0000,
    CMSG_MESSAGECHAT_ADDON_BATTLEGROUND               = 0x0D46,
    CMSG_MESSAGECHAT_ADDON_GUILD                      = 0x0544,
    CMSG_MESSAGECHAT_ADDON_OFFICER                    = 0x3954,
    CMSG_MESSAGECHAT_ADDON_PARTY                      = 0x0546,
    CMSG_MESSAGECHAT_ADDON_RAID                       = 0x1D56,
    CMSG_MESSAGECHAT_ADDON_WHISPER                    = 0x2146,
    CMSG_MESSAGECHAT_AFK                              = 0x0D44,
    CMSG_MESSAGECHAT_BATTLEGROUND                     = 0x2156,
    CMSG_MESSAGECHAT_CHANNEL                          = 0x1D44,
    CMSG_MESSAGECHAT_DND                              = 0x2946,
    CMSG_MESSAGECHAT_EMOTE                            = 0x1156,
    CMSG_MESSAGECHAT_GUILD                            = 0x3956,
    CMSG_MESSAGECHAT_OFFICER                          = 0x1946,
    CMSG_MESSAGECHAT_PARTY                            = 0x1D46,
    CMSG_MESSAGECHAT_RAID                             = 0x2D44,
    CMSG_MESSAGECHAT_RAID_WARNING                     = 0x0944,
    CMSG_MESSAGECHAT_SAY                              = 0x1154,
    CMSG_MESSAGECHAT_WHISPER                          = 0x0D56,
    CMSG_MESSAGECHAT_YELL                             = 0x3544,
    CMSG_MINIGAME_MOVE                                = 0x2A34,
    CMSG_MOUNTSPECIAL_ANIM                            = 0x2807,
    CMSG_MOVE_CHARM_TELEPORT_CHEAT                    = 0x0000,
    CMSG_MOVE_CHNG_TRANSPORT                          = 0x3102,
    CMSG_MOVE_ENABLE_SWIM_TO_FLY_TRANS_ACK            = 0x0000,
    CMSG_MOVE_FALL_RESET                              = 0x310A,
    CMSG_MOVE_FEATHER_FALL_ACK                        = 0x3110,
    CMSG_MOVE_FORCE_FLIGHT_BACK_SPEED_CHANGE_ACK      = 0x310E,
    CMSG_MOVE_FORCE_FLIGHT_SPEED_CHANGE_ACK           = 0x7314,
    CMSG_MOVE_FORCE_PITCH_RATE_CHANGE_ACK             = 0x3100,
    CMSG_MOVE_FORCE_RUN_BACK_SPEED_CHANGE_ACK         = 0x3216,
    CMSG_MOVE_FORCE_RUN_SPEED_CHANGE_ACK              = 0x7818,
    CMSG_MOVE_FORCE_SWIM_BACK_SPEED_CHANGE_ACK        = 0x7A16,
    CMSG_MOVE_FORCE_SWIM_SPEED_CHANGE_ACK             = 0x7A10,
    CMSG_MOVE_FORCE_TURN_RATE_CHANGE_ACK              = 0x7316,
    CMSG_MOVE_FORCE_WALK_SPEED_CHANGE_ACK             = 0x7210,
    CMSG_MOVE_GRAVITY_DISABLE_ACK                     = 0x3118,
    CMSG_MOVE_GRAVITY_ENABLE_ACK                      = 0x700A,
    CMSG_MOVE_HOVER_ACK                               = 0x3318,
    CMSG_MOVE_KNOCK_BACK_ACK                          = 0x721C,
    CMSG_MOVE_NOT_ACTIVE_MOVER                        = 0x7A1A,
    CMSG_MOVE_SET_CAN_FLY                             = 0x720E,
    CMSG_MOVE_SET_CAN_FLY_ACK                         = 0x790C,
    CMSG_MOVE_SET_CAN_FLY_TO_SWIM_ACK                 = 0x3014, // old CMSG_MOVE_SET_CAN_TRANSITION_BETWEEN_SWIM_AND_FLY_ACK
    CMSG_MOVE_SET_COLLISION_HEIGHT_ACK                = 0x7114,
    CMSG_MOVE_SET_RELATIVE_POSITION                   = 0x0000,
    CMSG_MOVE_SET_VEHICLE_REC_ID_ACK                  = 0x0000,
    CMSG_MOVE_SPLINE_DONE                             = 0x790E,
    CMSG_MOVE_TIME_SKIPPED                            = 0x7A0A,
    CMSG_MOVE_TOGGLE_COLLISION_ACK                    = 0x0000,
    CMSG_MOVE_WATER_WALK_ACK                          = 0x3B00,
    CMSG_NAME_QUERY                                   = 0x2224,
    CMSG_NEW_SPELL_SLOT                               = 0x0000,
    CMSG_NEXT_CINEMATIC_CAMERA                        = 0x2014,
    CMSG_NPC_TEXT_QUERY                               = 0x4E24,
    CMSG_OBJECT_UPDATE_FAILED                         = 0x3808,
    CMSG_OBJECT_UPDATE_RESCUED                        = 0x3906,
    CMSG_OFFER_PETITION                               = 0x4817,
    CMSG_OPENING_CINEMATIC                            = 0x0A16,
    CMSG_OPEN_ITEM                                    = 0x6A34,
    CMSG_OPT_OUT_OF_LOOT                              = 0x6B16,
    CMSG_PAGE_TEXT_QUERY                              = 0x6614,
    CMSG_PARTY_SILENCE                                = 0x6B26,
    CMSG_PARTY_UNSILENCE                              = 0x4D24,
    CMSG_PETITION_BUY                                 = 0x4E05,
    CMSG_PETITION_QUERY                               = 0x4424,
    CMSG_PETITION_SHOWLIST                            = 0x4617,
    CMSG_PETITION_SHOW_SIGNATURES                     = 0x4F15,
    CMSG_PETITION_SIGN                                = 0x0E04,
    CMSG_PET_ABANDON                                  = 0x0C24,
    CMSG_PET_ACTION                                   = 0x0226,
    CMSG_PET_CANCEL_AURA                              = 0x4B25,
    CMSG_PET_CAST_SPELL                               = 0x6337,
    CMSG_PET_LEARN_TALENT                             = 0x6725,
    CMSG_PET_NAME_CACHE                               = 0x0000,
    CMSG_PET_NAME_QUERY                               = 0x6F24,
    CMSG_PET_RENAME                                   = 0x6406,
    CMSG_PET_SET_ACTION                               = 0x6904,
    CMSG_PET_SPELL_AUTOCAST                           = 0x2514,
    CMSG_PET_STOP_ATTACK                              = 0x6C14,
    CMSG_PING                                         = 0x444D,
    CMSG_PLAYED_TIME                                  = 0x0804,
    CMSG_PLAYER_DIFFICULTY_CHANGE                     = 0x0000,
    CMSG_PLAYER_LOGIN                                 = 0x05B1,
    CMSG_PLAYER_LOGOUT                                = 0x0000,
    CMSG_PLAYER_VEHICLE_ENTER                         = 0x2705,
    CMSG_PLAY_DANCE                                   = 0x6914,
    CMSG_PUSHQUESTTOPARTY                             = 0x4B14,
    CMSG_PVP_LOG_DATA                                 = 0x7308,
    CMSG_QUERY_BATTLEFIELD_STATE                      = 0x7202,
    CMSG_QUERY_GUILD_MEMBERS_FOR_RECIPE               = 0x1036,
    CMSG_QUERY_GUILD_MEMBER_RECIPES                   = 0x1037,
    CMSG_QUERY_GUILD_RECIPES                          = 0x3033,
    CMSG_QUERY_GUILD_REWARDS                          = 0x3012,
    CMSG_QUERY_GUILD_XP                               = 0x1237,
    CMSG_QUERY_INSPECT_ACHIEVEMENTS                   = 0x4D27,
    CMSG_QUERY_QUESTS_COMPLETED                       = 0x2317,
    CMSG_QUERY_TIME                                   = 0x0A36,
    CMSG_QUESTGIVER_ACCEPT_QUEST                      = 0x6B37,
    CMSG_QUESTGIVER_CANCEL                            = 0x0000,
    CMSG_QUESTGIVER_CHOOSE_REWARD                     = 0x2125,
    CMSG_QUESTGIVER_COMPLETE_QUEST                    = 0x0114,
    CMSG_QUESTGIVER_HELLO                             = 0x0D17,
    CMSG_QUESTGIVER_QUERY_QUEST                       = 0x2F14,
    CMSG_QUESTGIVER_QUEST_AUTOLAUNCH                  = 0x0000,
    CMSG_QUESTGIVER_REQUEST_REWARD                    = 0x2534,
    CMSG_QUESTGIVER_STATUS_MULTIPLE_QUERY             = 0x6305,
    CMSG_QUESTGIVER_STATUS_QUERY                      = 0x4407,
    CMSG_QUESTLOG_REMOVE_QUEST                        = 0x0D16,
    CMSG_QUESTLOG_SWAP_QUEST                          = 0x0000,
    CMSG_QUEST_CONFIRM_ACCEPT                         = 0x0D15,
    CMSG_QUEST_NPC_QUERY                              = 0x7302,
    CMSG_QUEST_POI_QUERY                              = 0x4037,
    CMSG_QUEST_QUERY                                  = 0x0D06,
    CMSG_RANDOMIZE_CHAR_NAME                          = 0x2413,
    CMSG_READY_FOR_ACCOUNT_DATA_TIMES                 = 0x2B16,
    CMSG_READ_ITEM                                    = 0x2F16,
    CMSG_REALM_SPLIT                                  = 0x2906,
    CMSG_RECLAIM_CORPSE                               = 0x4036,
    CMSG_REDIRECTION_AUTH_PROOF                       = 0x044D,
    CMSG_REFORGE_ITEM                                 = 0x331A,
    CMSG_REORDER_CHARACTERS                           = 0x0593,
    CMSG_REPAIR_ITEM                                  = 0x2917,
    CMSG_REPLACE_ACCOUNT_DATA                         = 0x0000,
    CMSG_REPOP_REQUEST                                = 0x6235,
    CMSG_REPORT_PVP_AFK                               = 0x6734,
    CMSG_REQUEST_ACCOUNT_DATA                         = 0x6505,
    CMSG_REQUEST_CATEGORY_COOLDOWNS                   = 0x7102,
    CMSG_REQUEST_CEMETERY_LIST                        = 0x720A,
    CMSG_REQUEST_HOTFIX                               = 0x2401,
    CMSG_REQUEST_INSPECT_RATED_BG_STATS               = 0x3010,
    CMSG_REQUEST_PARTY_MEMBER_STATS                   = 0x0C04,
    CMSG_REQUEST_PET_INFO                             = 0x4924,
    CMSG_REQUEST_PVP_OPTIONS_ENABLED                  = 0x24A1,
    CMSG_REQUEST_PVP_REWARDS                          = 0x780C,
    CMSG_REQUEST_RAID_INFO                            = 0x2F26,
    CMSG_REQUEST_RATED_BG_INFO                        = 0x2423,
    CMSG_REQUEST_RATED_BG_STATS                       = 0x05B3,
    CMSG_REQUEST_RESEARCH_HISTORY                     = 0x3306,
    CMSG_REQUEST_VEHICLE_EXIT                         = 0x2B35,
    CMSG_REQUEST_VEHICLE_NEXT_SEAT                    = 0x4434,
    CMSG_REQUEST_VEHICLE_PREV_SEAT                    = 0x4C04,
    CMSG_REQUEST_VEHICLE_SWITCH_SEAT                  = 0x4C14,
    CMSG_RESET_FACTION_CHEAT                          = 0x4469,
    CMSG_RESET_INSTANCES                              = 0x6E14,
    CMSG_RESURRECT_RESPONSE                           = 0x6827,
    CMSG_RETURN_TO_GRAVEYARD                          = 0x301E,
    CMSG_ROLE_POLL_BEGIN                              = 0x0430,
    CMSG_SAVE_CUF_PROFILES                            = 0x730E,
    CMSG_SAVE_PLAYER                                  = 0x0000,
    CMSG_SEARCH_LFG_JOIN                              = 0x0000,
    CMSG_SEARCH_LFG_LEAVE                             = 0x0000,
    CMSG_SELF_RES                                     = 0x6115,
    CMSG_SELL_ITEM                                    = 0x4E15,
    CMSG_SEND_MAIL                                    = 0x0523,
    CMSG_SEND_SOR_REQUEST_VIA_ADDRESS                 = 0x0420,
    CMSG_SEND_SOR_REQUEST_VIA_BNET_ACCOUNT_ID         = 0x0482,
    CMSG_SERVERTIME                                   = 0x0000,
    CMSG_SETDEATHBINDPOINT                            = 0x0000,
    CMSG_SETSHEATHED                                  = 0x4326,
    CMSG_SET_ACTIONBAR_TOGGLES                        = 0x2506,
    CMSG_SET_ACTION_BUTTON                            = 0x6F06,
    CMSG_SET_ACTIVE_MOVER                             = 0x3314,
    CMSG_SET_ACTIVE_VOICE_CHANNEL                     = 0x4305,
    CMSG_SET_ALLOW_LOW_LEVEL_RAID1                    = 0x4435,
    CMSG_SET_ALLOW_LOW_LEVEL_RAID2                    = 0x0536,
    CMSG_SET_CHANNEL_WATCH                            = 0x4517,
    CMSG_SET_CONTACT_NOTES                            = 0x6135,
    CMSG_SET_CURRENCY_FLAGS                           = 0x7306,
    CMSG_SET_EVERYONE_IS_ASSISTANT                    = 0x2530,
    CMSG_SET_FACTION_ATWAR                            = 0x0706,
    CMSG_SET_FACTION_CHEAT                            = 0x0000,
    CMSG_SET_FACTION_INACTIVE                         = 0x0E37,
    CMSG_SET_GUILD_BANK_TEXT                          = 0x3023,
    CMSG_SET_LFG_COMMENT                              = 0x0000,
    CMSG_SET_PET_SLOT                                 = 0x3A04,
    CMSG_SET_PLAYER_DECLINED_NAMES                    = 0x6316,
    CMSG_SET_PREFERED_CEMETERY                        = 0x311E,
    CMSG_SET_PRIMARY_TALENT_TREE                      = 0x4524,
    CMSG_SET_RELATIVE_POSITION                        = 0x311A,
    CMSG_SET_SAVED_INSTANCE_EXTEND                    = 0x6706,
    CMSG_SET_SELECTION                                = 0x0506,
    CMSG_SET_SKILL_CHEAT                              = 0x0000,
    CMSG_SET_TAXI_BENCHMARK_MODE                      = 0x4314,
    CMSG_SET_TITLE                                    = 0x2117,
    CMSG_SET_TRADE_CURRENCY                           = 0x3312,
    CMSG_SET_TRADE_GOLD                               = 0x3008,
    CMSG_SET_TRADE_ITEM                               = 0x7B0C,
    CMSG_SET_VEHICLE_REC_ID_ACK                       = 0x3108,
    CMSG_SET_WATCHED_FACTION                          = 0x2434,
    CMSG_SHOWING_CLOAK                                = 0x4135,
    CMSG_SHOWING_HELM                                 = 0x0735,
    CMSG_SOCKET_GEMS                                  = 0x2F04,
    CMSG_SPELLCLICK                                   = 0x0805,
    CMSG_SPIRIT_HEALER_ACTIVATE                       = 0x2E26,
    CMSG_SPLIT_ITEM                                   = 0x0F17,
    CMSG_STANDSTATECHANGE                             = 0x0535,
    CMSG_START_QUEST                                  = 0x0000,
    CMSG_STOP_DANCE                                   = 0x2907,
    CMSG_STORE_LOOT_IN_SLOT                           = 0x0000,
    CMSG_SUBMIT_BUG                                   = 0x2520,
    CMSG_SUBMIT_COMPLAIN                              = 0x2501,
    CMSG_SUGGESTION_SUBMIT                            = 0x2512,
    CMSG_SUMMON_RESPONSE                              = 0x6F27,
    CMSG_SUSPEND_TOKEN                                = 0x046D,
    CMSG_SWAP_INV_ITEM                                = 0x2614,
    CMSG_SWAP_ITEM                                    = 0x6326,
    CMSG_SYNC_DANCE                                   = 0x0036,
    CMSG_TAXICLEARALLNODES                            = 0x0000,
    CMSG_TAXIENABLEALLNODES                           = 0x0000,
    CMSG_TAXINODE_STATUS_QUERY                        = 0x2F25,
    CMSG_TAXIQUERYAVAILABLENODES                      = 0x6C06,
    CMSG_TAXISHOWNODES                                = 0x0000,
    CMSG_TELEPORT_TO_UNIT                             = 0x4206,
    CMSG_TEXT_EMOTE                                   = 0x2E24,
    CMSG_TIME_ADJUSTMENT_RESPONSE                     = 0x3818,
    CMSG_TIME_SYNC_RESP                               = 0x3B0C,
    CMSG_TIME_SYNC_RESP_FAILED                        = 0x710A,
    CMSG_TOGGLE_PVP                                   = 0x6815,
    CMSG_TOTEM_DESTROYED                              = 0x4207,
    CMSG_TRAINER_BUY_SPELL                            = 0x4415,
    CMSG_TRAINER_LIST                                 = 0x2336,
    CMSG_TRANSMOGRIFY_ITEMS                           = 0x3B0E,
    CMSG_TRIGGER_CINEMATIC_CHEAT                      = 0x0000,
    CMSG_TURN_IN_PETITION                             = 0x0B27,
    CMSG_TUTORIAL_CLEAR                               = 0x6515,
    CMSG_TUTORIAL_FLAG                                = 0x6C26,
    CMSG_TUTORIAL_RESET                               = 0x2726,
    CMSG_UNACCEPT_TRADE                               = 0x391A,
    CMSG_UNLEARN_SKILL                                = 0x6106,
    CMSG_UNLEARN_SPECIALIZATION                       = 0x3210,
    CMSG_UNREGISTER_ALL_ADDON_PREFIXES                = 0x3D54,
    CMSG_UPDATE_ACCOUNT_DATA                          = 0x4736,
    CMSG_UPDATE_MISSILE_TRAJECTORY                    = 0x781E,
    CMSG_UPDATE_PROJECTILE_POSITION                   = 0x0E24,
    CMSG_USED_FOLLOW                                  = 0x7912,
    CMSG_USE_ITEM                                     = 0x2C06,
    CMSG_VIOLENCE_LEVEL                               = 0x7816,
    CMSG_VOICE_SESSION_ENABLE                         = 0x2314,
    CMSG_VOID_STORAGE_QUERY                           = 0x790A,
    CMSG_VOID_STORAGE_TRANSFER                        = 0x380E,
    CMSG_VOID_STORAGE_UNLOCK                          = 0x7B14,
    CMSG_VOID_SWAP_ITEM                               = 0x3204,
    CMSG_WARDEN_DATA                                  = 0x25A2,
    CMSG_WARGAME_ACCEPT                               = 0x2410,
    CMSG_WARGAME_START                                = 0x05A0,
    CMSG_WHO                                          = 0x6C15,
    CMSG_WHOIS                                        = 0x6B05,
    CMSG_WORLD_STATE_UI_TIMER_UPDATE                  = 0x4605,
    CMSG_WORLD_TELEPORT                               = 0x24B2,
    CMSG_WRAP_ITEM                                    = 0x4F06,
    CMSG_ZONEUPDATE                                   = 0x4F37,
    MSG_AUCTION_HELLO                                 = 0x2307,
    MSG_CHANNEL_START                                 = 0x0A15, // SMSG only?
    MSG_CHANNEL_UPDATE                                = 0x2417, // SMSG only?
    MSG_CORPSE_QUERY                                  = 0x4336,
    MSG_GM_BIND_OTHER                                 = 0x0000,
    MSG_GM_SHOWLABEL                                  = 0x0000,
    MSG_GM_SUMMON                                     = 0x0000,
    MSG_INSPECT_ARENA_TEAMS                           = 0x2704,
    MSG_LIST_STABLED_PETS                             = 0x0834,
    MSG_MINIMAP_PING                                  = 0x6635,
    MSG_MOVE_CHARM_TELEPORT_CHEAT                     = 0x7A08,
    MSG_MOVE_FALL_LAND                                = 0x380A,
    MSG_MOVE_FEATHER_FALL                             = 0x0000,
    MSG_MOVE_HEARTBEAT                                = 0x3914,
    MSG_MOVE_HOVER                                    = 0x0000,
    MSG_MOVE_JUMP                                     = 0x7A06,
    MSG_MOVE_SET_ALL_SPEED_CHEAT                      = 0x0000,
    MSG_MOVE_SET_COLLISION_HEIGHT                     = 0x0000,
    MSG_MOVE_SET_FACING                               = 0x7914,
    MSG_MOVE_SET_FLIGHT_SPEED_CHEAT                   = 0x0000,
    MSG_MOVE_SET_PITCH                                = 0x7312,
    MSG_MOVE_SET_RAW_POSITION_ACK                     = 0x0000,
    MSG_MOVE_SET_RUN_BACK_SPEED_CHEAT                 = 0x0000,
    SMSG_MOVE_SET_RUN_MODE                            = 0x791A,
    MSG_MOVE_SET_RUN_SPEED_CHEAT                      = 0x0000,
    MSG_MOVE_SET_SWIM_BACK_SPEED_CHEAT                = 0x0000,
    MSG_MOVE_SET_SWIM_SPEED_CHEAT                     = 0x0000,
    MSG_MOVE_SET_TURN_RATE_CHEAT                      = 0x0000,
    SMSG_MOVE_SET_WALK_MODE                           = 0x7002,
    MSG_MOVE_SET_WALK_SPEED_CHEAT                     = 0x0000,
    MSG_MOVE_START_ASCEND                             = 0x390A,
    MSG_MOVE_START_BACKWARD                           = 0x330A,
    MSG_MOVE_START_DESCEND                            = 0x3800,
    MSG_MOVE_START_FORWARD                            = 0x7814,
    MSG_MOVE_START_PITCH_DOWN                         = 0x3908,
    MSG_MOVE_START_PITCH_UP                           = 0x3304,
    MSG_MOVE_START_STRAFE_LEFT                        = 0x3A16,
    MSG_MOVE_START_STRAFE_RIGHT                       = 0x3A02,
    SMSG_MOVE_START_SWIM                              = 0x3206,
    MSG_MOVE_START_TURN_LEFT                          = 0x700C,
    MSG_MOVE_START_TURN_RIGHT                         = 0x7000,
    MSG_MOVE_STOP                                     = 0x320A,
    MSG_MOVE_STOP_ASCEND                              = 0x7B00,
    MSG_MOVE_STOP_PITCH                               = 0x7216,
    MSG_MOVE_STOP_STRAFE                              = 0x3002,
    SMSG_MOVE_STOP_SWIM                               = 0x3802,
    MSG_MOVE_STOP_TURN                                = 0x331E,
    SMSG_MOVE_TELEPORT                                = 0x55A0,
    CMSG_MOVE_TELEPORT_ACK                            = 0x390C,
    MSG_MOVE_TELEPORT_CHEAT                           = 0x3A10,
    MSG_MOVE_TIME_SKIPPED                             = 0x19B3,
    MSG_MOVE_TOGGLE_COLLISION_CHEAT                   = 0x7B04,
    MSG_MOVE_TOGGLE_FALL_LOGGING                      = 0x0000,
    MSG_MOVE_TOGGLE_LOGGING                           = 0x0000,
    MSG_MOVE_UPDATE_CAN_FLY                           = 0x0000,
    SMSG_MOVE_UPDATE_FLIGHT_SPEED                     = 0x30B1,
    MSG_MOVE_UPDATE_MOUSE                             = 0x0000,
    SMSG_MOVE_UPDATE_RUN_SPEED                        = 0x14A6,
    MSG_MOVE_WORLDPORT_ACK                            = 0x2411,
    MSG_NOTIFY_PARTY_SQUELCH                          = 0x4D06,
    MSG_PARTY_ASSIGNMENT                              = 0x0424,
    MSG_PETITION_DECLINE                              = 0x4905,
    MSG_PETITION_RENAME                               = 0x4005,
    MSG_PVP_LOG_DATA                                  = 0x0000,
    MSG_QUERY_NEXT_MAIL_TIME                          = 0x0F04,
    MSG_QUEST_PUSH_RESULT                             = 0x4515,
    MSG_RAID_READY_CHECK                              = 0x2304,
    MSG_RAID_READY_CHECK_CONFIRM                      = 0x4F05,
    MSG_RAID_READY_CHECK_FINISHED                     = 0x2E15,
    MSG_RAID_TARGET_UPDATE                            = 0x2C36,
    MSG_RANDOM_ROLL                                   = 0x0905,
    MSG_SAVE_GUILD_EMBLEM                             = 0x2404,
    MSG_SET_DUNGEON_DIFFICULTY                        = 0x4925,
    MSG_SET_RAID_DIFFICULTY                           = 0x0614,
    MSG_START_MOVE_FORWARD                            = 0x0000,
    MSG_TABARDVENDOR_ACTIVATE                         = 0x6926,
    MSG_TALENT_WIPE_CONFIRM                           = 0x0107,
    MSG_VERIFY_CONNECTIVITY                           = 0x4F57,
    SMSG_ACCOUNT_DATA_TIMES                           = 0x4B05,
    SMSG_ACCOUNT_INFO_RESPONSE                        = 0x10A7,
    SMSG_ACCOUNT_RESTRICTED_WARNING                   = 0x51A7,
    SMSG_ACHIEVEMENT_DELETED                          = 0x6A16,
    SMSG_ACHIEVEMENT_EARNED                           = 0x4405,
    SMSG_ACTION_BUTTONS                               = 0x38B5,
    SMSG_ACTIVATETAXIREPLY                            = 0x6A37,
    SMSG_ADDON_INFO                                   = 0x2C14,
    SMSG_ADD_RUNE_POWER                               = 0x6915,
    SMSG_AI_REACTION                                  = 0x0637,
    SMSG_ALL_ACHIEVEMENT_DATA                         = 0x58B1,
    SMSG_AREA_SPIRIT_HEALER_TIME                      = 0x0734,
    SMSG_AREA_TRIGGER_MESSAGE                         = 0x4505,
    SMSG_AREA_TRIGGER_MOVEMENT_UPDATE                 = 0x3DB1,
    SMSG_ARENA_ERROR                                  = 0x2D17,
    SMSG_ARENA_UNIT_DESTROYED                         = 0x2637,
    SMSG_ARENA_TEAM_CHANGE_FAILED_QUEUED              = 0x6E34,
    SMSG_ARENA_TEAM_COMMAND_RESULT                    = 0x39B3,
    SMSG_ARENA_TEAM_EVENT                             = 0x0617,
    SMSG_ARENA_TEAM_INVITE                            = 0x0F36,
    SMSG_ARENA_TEAM_QUERY_RESPONSE                    = 0x6336,
    SMSG_ARENA_TEAM_ROSTER                            = 0x2717,
    SMSG_ARENA_TEAM_STATS                             = 0x4425,
    SMSG_ATTACKERSTATEUPDATE                          = 0x0B25,
    SMSG_ATTACKSTART                                  = 0x2D15,
    SMSG_ATTACKSTOP                                   = 0x0934,
    SMSG_ATTACKSWING_BADFACING                        = 0x0B36,
    SMSG_ATTACKSWING_CANT_ATTACK                      = 0x0016,
    SMSG_ATTACKSWING_DEADTARGET                       = 0x2B26,
    SMSG_ATTACKSWING_NOTINRANGE                       = 0x6C07,
    SMSG_AUCTION_BIDDER_LIST_RESULT                   = 0x0027,
    SMSG_AUCTION_BIDDER_NOTIFICATION                  = 0x4E27,
    SMSG_AUCTION_COMMAND_RESULT                       = 0x4C25,
    SMSG_AUCTION_LIST_PENDING_SALES                   = 0x6A27,
    SMSG_AUCTION_LIST_RESULT                          = 0x6637,
    SMSG_AUCTION_OWNER_LIST_RESULT                    = 0x6C34,
    SMSG_AUCTION_OWNER_NOTIFICATION                   = 0x4116,
    SMSG_AUCTION_REMOVED_NOTIFICATION                 = 0x2334,
    SMSG_AURACASTLOG                                  = 0x0000,
    SMSG_AURA_POINTS_DEPLETED                         = 0x7CB7,
    SMSG_AURA_UPDATE                                  = 0x4707,
    SMSG_AURA_UPDATE_ALL                              = 0x6916,
    SMSG_AUTH_CHALLENGE                               = 0x4542,
    SMSG_AUTH_RESPONSE                                = 0x5DB6,
    SMSG_AVAILABLE_VOICE_CHANNEL                      = 0x2E16,
    SMSG_AVERAGE_ITEM_LEVEL_INFORM                    = 0x5DA7,
    SMSG_COMPLETED_ARTIFACTS                          = 0x0000,
    SMSG_BARBER_SHOP_RESULT                           = 0x6125,
    SMSG_BATTLEFIELD_LIST                             = 0x71B5,
    SMSG_BATTLEFIELD_MGR_EJECTED                      = 0x7DB7,
    SMSG_BATTLEFIELD_MGR_EJECT_PENDING                = 0x34A2,
    SMSG_BATTLEFIELD_MGR_ENTERED                      = 0x5CA0,
    SMSG_BATTLEFIELD_MGR_ENTRY_INVITE                 = 0x34B3,
    SMSG_BATTLEFIELD_MGR_EXIT_REQUEST                 = 0x51B1,
    SMSG_BATTLEFIELD_MGR_QUEUE_INVITE                 = 0x15A6,
    SMSG_BATTLEFIELD_MGR_QUEUE_REQUEST_RESPONSE       = 0x79B6,
    SMSG_BATTLEFIELD_MGR_STATE_CHANGE                 = 0x35B4,
    SMSG_BATTLEFIELD_PLAYER_POSITIONS                 = 0x58B4,
    SMSG_BATTLEFIELD_PORT_DENIED                      = 0x35A3,
    SMSG_BATTLEFIELD_RATED_INFO                       = 0x54A3,
    SMSG_BATTLEFIELD_STATUS                           = 0x7DA1,
    SMSG_BATTLEFIELD_STATUS_QUEUED                    = 0x35A1,
    SMSG_BATTLEFIELD_STATUS_ACTIVE                    = 0x74A4,
    SMSG_BATTLEFIELD_STATUS_NEEDCONFIRMATION          = 0x59A0,
    SMSG_BATTLEFIELD_STATUS_WAITFORGROUPS             = 0x75A2,
    SMSG_BATTLEFIELD_STATUS_FAILED                    = 0x71A7,
    SMSG_BATTLEGROUND_INFO_THROTTLED                  = 0x34B2,
    SMSG_BATTLEGROUND_PLAYER_JOINED                   = 0x50B0,
    SMSG_BATTLEGROUND_PLAYER_LEFT                     = 0x59A6,
    SMSG_BINDER_CONFIRM                               = 0x2835,
    SMSG_BINDPOINTUPDATE                              = 0x0527,
    SMSG_BINDZONEREPLY                                = 0x0000,
    SMSG_BREAK_TARGET                                 = 0x0105,
    SMSG_BUY_BANK_SLOT_RESULT                         = 0x4806,
    SMSG_BUY_FAILED                                   = 0x6435,
    SMSG_BUY_ITEM                                     = 0x0F26,
    SMSG_CALENDAR_ACTION_PENDING                      = 0x0000,
    SMSG_CALENDAR_ARENA_TEAM                          = 0x0615,
    SMSG_CALENDAR_CLEAR_PENDING_ACTION                = 0x2106,
    SMSG_CALENDAR_COMMAND_RESULT                      = 0x6F36,
    SMSG_CALENDAR_EVENT_INVITE                        = 0x4E16,
    SMSG_CALENDAR_EVENT_INVITE_ALERT                  = 0x2A05,
    SMSG_CALENDAR_EVENT_INVITE_NOTES                  = 0x0E17,
    SMSG_CALENDAR_EVENT_INVITE_NOTES_ALERT            = 0x2535,
    SMSG_CALENDAR_EVENT_INVITE_REMOVED                = 0x0725,
    SMSG_CALENDAR_EVENT_INVITE_REMOVED_ALERT          = 0x2617,
    SMSG_CALENDAR_EVENT_INVITE_STATUS_ALERT           = 0x6625,
    SMSG_CALENDAR_EVENT_MODERATOR_STATUS_ALERT        = 0x6B06,
    SMSG_CALENDAR_EVENT_REMOVED_ALERT                 = 0x6D35,
    SMSG_CALENDAR_EVENT_STATUS                        = 0x2A27,
    SMSG_CALENDAR_EVENT_UPDATED_ALERT                 = 0x0907,
    SMSG_CALENDAR_FILTER_GUILD                        = 0x4A26,
    SMSG_CALENDAR_RAID_LOCKOUT_ADDED                  = 0x2305,
    SMSG_CALENDAR_RAID_LOCKOUT_REMOVED                = 0x2E25,
    SMSG_CALENDAR_RAID_LOCKOUT_UPDATED                = 0x4636,
    SMSG_CALENDAR_SEND_CALENDAR                       = 0x6805,
    SMSG_CALENDAR_SEND_EVENT                          = 0x0C35,
    SMSG_CALENDAR_SEND_NUM_PENDING                    = 0x0C17,
    SMSG_CALENDAR_UPDATE_INVITE_LIST                  = 0x0000,
    SMSG_CAMERA_SHAKE                                 = 0x4214,
    SMSG_CANCEL_AUTO_REPEAT                           = 0x6436,
    SMSG_CANCEL_COMBAT                                = 0x4F04,
    SMSG_CAST_FAILED                                  = 0x4D16,
    SMSG_CHANNEL_LIST                                 = 0x2214,
    SMSG_CHANNEL_MEMBER_COUNT                         = 0x6414,
    SMSG_CHANNEL_NOTIFY                               = 0x0825,
    SMSG_CHARACTER_LOGIN_FAILED                       = 0x4417,
    SMSG_CHAR_CREATE                                  = 0x2D05,
    SMSG_CHAR_CUSTOMIZE                               = 0x4F16,
    SMSG_CHAR_DELETE                                  = 0x0304,
    SMSG_CHAR_ENUM                                    = 0x10B0,
    SMSG_CHAR_FACTION_CHANGE                          = 0x4C06,
    SMSG_CHAR_RENAME                                  = 0x2024,
    SMSG_CHAT_IGNORED_ACCOUNT_MUTED                   = 0x15A4,
    SMSG_CHAT_NOT_IN_PARTY                            = 0x6A14,
    SMSG_CHAT_PLAYER_AMBIGUOUS                        = 0x2F34,
    SMSG_CHAT_PLAYER_NOT_FOUND                        = 0x2526,
    SMSG_CHAT_RESTRICTED                              = 0x6536,
    SMSG_CHAT_SERVER_DISCONNECTED                     = 0x6D34,
    SMSG_CHAT_SERVER_RECONNECTED                      = 0x6905,
    SMSG_CHAT_WRONG_FACTION                           = 0x6724,
    SMSG_CHECK_FOR_BOTS                               = 0x0000,
    SMSG_CLEAR_BOSS_EMOTES                            = 0x19A3,
    SMSG_CLEAR_COOLDOWN                               = 0x0627,
    SMSG_CLEAR_COOLDOWNS                              = 0x59B4,
    SMSG_CLEAR_FAR_SIGHT_IMMEDIATE                    = 0x2A04,
    SMSG_CLEAR_TARGET                                 = 0x4B26,
    SMSG_CLIENTCACHE_VERSION                          = 0x2734,
    SMSG_CLIENT_CONTROL_UPDATE                        = 0x2837,
    SMSG_COMBAT_EVENT_FAILED                          = 0x2B07,
    SMSG_COMBAT_LOG_MULTIPLE                          = 0x0000,
    SMSG_COMMENTATOR_MAP_INFO                         = 0x0327,
    SMSG_COMMENTATOR_PARTY_INFO                       = 0x38B0,
    SMSG_COMMENTATOR_PLAYER_INFO                      = 0x2F36,
    SMSG_COMMENTATOR_SKIRMISH_QUEUE_RESULT1           = 0x2126,
    SMSG_COMMENTATOR_SKIRMISH_QUEUE_RESULT2           = 0x6814,
    SMSG_COMMENTATOR_STATE_CHANGED                    = 0x0737,
    SMSG_COMPLAIN_RESULT                              = 0x6D24,
    SMSG_COMPRESSED_MOVES                             = 0x0517,
    SMSG_COMSAT_CONNECT_FAIL                          = 0x6317,
    SMSG_COMSAT_DISCONNECT                            = 0x0316,
    SMSG_COMSAT_RECONNECT_TRY                         = 0x4D35,
    SMSG_CONTACT_LIST                                 = 0x6017,
    SMSG_CONVERT_RUNE                                 = 0x4F14,
    SMSG_COOLDOWN_CHEAT                               = 0x4537,
    SMSG_COOLDOWN_EVENT                               = 0x4F26,
    SMSG_CORPSE_MAP_POSITION_QUERY_RESPONSE           = 0x0E35,
    SMSG_CORPSE_NOT_IN_INSTANCE                       = 0x2A14,
    SMSG_CORPSE_RECLAIM_DELAY                         = 0x0D34,
    SMSG_CREATURE_QUERY_RESPONSE                      = 0x6024,
    SMSG_CRITERIA_DELETED                             = 0x2915,
    SMSG_CRITERIA_UPDATE                              = 0x6E37,
    SMSG_CROSSED_INEBRIATION_THRESHOLD                = 0x2036,
    SMSG_CURRENCY_LOOT_REMOVED                        = 0x1DB4,
    SMSG_CURRENCY_LOOT_RESTORED                       = 0x30A0,
    SMSG_CUSTOM_LOAD_SCREEN                           = 0x1DB6,
    SMSG_DAMAGE_CALC_LOG                              = 0x2436,
    SMSG_DAMAGE_DONE_OBSOLETE                         = 0x0000,
    SMSG_DANCE_QUERY_RESPONSE                         = 0x2F06,
    SMSG_DB_REPLY                                     = 0x38A4,
    SMSG_DEATH_RELEASE_LOC                            = 0x2F07,
    SMSG_DEBUG_RUNE_REGEN                             = 0x31B3,
    SMSG_DEFENSE_MESSAGE                              = 0x0314,
    SMSG_DESTROY_OBJECT                               = 0x4724,
    SMSG_DESTRUCTIBLE_BUILDING_DAMAGE                 = 0x4825,
    SMSG_DIFFERENT_INSTANCE_FROM_PARTY                = 0x15B1,
    SMSG_DISENCHANT_CREDIT                            = 0x55A2,
    SMSG_DISMOUNT                                     = 0x2135,
    SMSG_DISMOUNTRESULT                               = 0x0D25,
    SMSG_DISPEL_FAILED                                = 0x0307,
    SMSG_DISPLAY_GAME_ERROR                           = 0x31A6,
    SMSG_DONT_AUTO_PUSH_SPELLS_TO_ACTION_BAR          = 0x38A2,
    SMSG_DROP_NEW_CONNECTION                          = 0x4D40,
    SMSG_DUEL_COMPLETE                                = 0x2527,
    SMSG_DUEL_COUNTDOWN                               = 0x4836,
    SMSG_DUEL_INBOUNDS                                = 0x0A27,
    SMSG_DUEL_OUTOFBOUNDS                             = 0x0C26,
    SMSG_DUEL_REQUESTED                               = 0x4504,
    SMSG_DUEL_WINNER                                  = 0x2D36,
    SMSG_DUMP_RIDE_TICKETS_RESPONSE                   = 0x11A3,
    SMSG_DURABILITY_DAMAGE_DEATH                      = 0x4C27,
    SMSG_ECHO_PARTY_SQUELCH                           = 0x0814,
    SMSG_EMOTE                                        = 0x0A34,
    SMSG_ENABLE_BARBER_SHOP                           = 0x2D16,
    SMSG_ENCHANTMENTLOG                               = 0x6035,
    SMSG_ENVIRONMENTALDAMAGELOG                       = 0x6C05,
    SMSG_EQUIPMENT_SET_LIST                           = 0x2E04,
    SMSG_EQUIPMENT_SET_SAVED                          = 0x2216,
    SMSG_EQUIPMENT_SET_USE_RESULT                     = 0x2424,
    SMSG_EXPECTED_SPAM_RECORDS                        = 0x4D36,
    SMSG_EXPLORATION_EXPERIENCE                       = 0x6716,
    SMSG_FAILED_PLAYER_CONDITION                      = 0x19A4,
    SMSG_FEATURE_SYSTEM_STATUS                        = 0x3DB7,
    SMSG_FEIGN_DEATH_RESISTED                         = 0x0D05,
    SMSG_FISH_ESCAPED                                 = 0x2205,
    SMSG_FISH_NOT_HOOKED                              = 0x0A17,
    SMSG_FLIGHT_SPLINE_SYNC                           = 0x0924,
    SMSG_FLOOD_DETECTED                               = 0x0542,
    SMSG_FORCEACTIONSHOW                              = 0x0000,
    SMSG_FORCED_DEATH_UPDATE                          = 0x2606,
    SMSG_FORCE_DISPLAY_UPDATE                         = 0x0000,
    SMSG_FORCE_SEND_QUEUED_PACKETS                    = 0x0140,
    SMSG_FORCE_SET_VEHICLE_REC_ID                     = 0x70A1,
    SMSG_FORGE_MASTER_SET                             = 0x70B7,
    SMSG_FRIEND_STATUS                                = 0x0717,
    SMSG_GAMEOBJECT_CUSTOM_ANIM                       = 0x4936,
    SMSG_GAMEOBJECT_DESPAWN_ANIM                      = 0x6735,
    SMSG_GAMEOBJECT_PAGETEXT                          = 0x2925,
    SMSG_GAMEOBJECT_QUERY_RESPONSE                    = 0x0915,
    SMSG_GAMEOBJECT_RESET_STATE                       = 0x2A16,
    SMSG_GAMESPEED_SET                                = 0x4E34,
    SMSG_GAMETIME_SET                                 = 0x0014,
    SMSG_GAMETIME_UPDATE                              = 0x4127,
    SMSG_GAME_EVENT_DEBUG_LOG                         = 0x31A7,
    SMSG_GAME_OBJECT_ACTIVATE_ANIM_KIT                = 0x14A3,
    SMSG_GMRESPONSE_DB_ERROR                          = 0x0006,
    SMSG_GMRESPONSE_RECEIVED                          = 0x2E34,
    SMSG_GMRESPONSE_STATUS_UPDATE                     = 0x0A04,
    SMSG_GMTICKET_CREATE                              = 0x2107,
    SMSG_GMTICKET_DELETETICKET                        = 0x6D17,
    SMSG_GMTICKET_GETTICKET                           = 0x2C15,
    SMSG_GMTICKET_SYSTEMSTATUS                        = 0x0D35,
    SMSG_GMTICKET_UPDATETEXT                          = 0x6535,
    SMSG_GM_MESSAGECHAT                               = 0x6434,
    SMSG_GM_PLAYER_INFO                               = 0x4A15,
    SMSG_GM_TICKET_STATUS_UPDATE                      = 0x2C25,
    SMSG_GODMODE                                      = 0x0405,
    SMSG_GOSSIP_COMPLETE                              = 0x0806,
    SMSG_GOSSIP_MESSAGE                               = 0x2035,
    SMSG_GOSSIP_POI                                   = 0x4316,
    SMSG_GROUPACTION_THROTTLED                        = 0x6524,
    SMSG_GROUP_CANCEL                                 = 0x4D25,
    SMSG_GROUP_DECLINE                                = 0x6835,
    SMSG_GROUP_DESTROYED                              = 0x2207,
    SMSG_GROUP_INVITE                                 = 0x31B2,
    SMSG_GROUP_LIST                                   = 0x4C24,
    SMSG_GROUP_SET_LEADER                             = 0x0526,
    SMSG_GROUP_SET_ROLE                               = 0x39A6,
    SMSG_GROUP_UNINVITE                               = 0x0A07,
    SMSG_GUILD_ACHIEVEMENT_DATA                       = 0x54B7,
    SMSG_GUILD_ACHIEVEMENT_DELETED                    = 0x35A0,
    SMSG_GUILD_ACHIEVEMENT_EARNED                     = 0x50B5,
    SMSG_GUILD_ACHIEVEMENT_MEMBERS                    = 0x38A5,
    SMSG_GUILD_BANK_LIST                              = 0x78A5,
    SMSG_GUILD_BANK_LOG_QUERY_RESULT                  = 0x30B2,
    SMSG_GUILD_BANK_MONEY_WITHDRAWN                   = 0x5DB4,
    SMSG_GUILD_BANK_QUERY_TEXT_RESULT                 = 0x75A3,
    SMSG_GUILD_CANCEL                                 = 0x0000,
    SMSG_GUILD_CHALLENGE_COMPLETED                    = 0x39A3,
    SMSG_GUILD_CHALLENGE_UPDATED                      = 0x18B1,
    SMSG_GUILD_CHANGE_NAME_RESULT                     = 0x3CB1,
    SMSG_GUILD_COMMAND_RESULT                         = 0x7DB3,
    SMSG_GUILD_COMMAND_RESULT_2                       = 0x2707,
    SMSG_GUILD_CRITERIA_DATA                          = 0x14B4,
    SMSG_GUILD_CRITERIA_DELETED                       = 0x55B1,
    SMSG_GUILD_DECLINE                                = 0x2C07,
    SMSG_GUILD_EVENT                                  = 0x0705,
    SMSG_GUILD_EVENT_LOG_QUERY_RESULT                 = 0x10B2,
    SMSG_GUILD_FLAGGED_FOR_RENAME                     = 0x30B6,
    SMSG_GUILD_INVITE                                 = 0x14A2,
    SMSG_GUILD_INVITE_CANCEL                          = 0x0606,
    SMSG_GUILD_KNOWN_RECIPES                          = 0x0000,
    SMSG_GUILD_MAX_DAILY_XP                           = 0x79B5,
    SMSG_GUILD_MEMBERS_FOR_RECIPE                     = 0x1CB7,
    SMSG_GUILD_MEMBER_DAILY_RESET                     = 0x10A5,
    SMSG_GUILD_MEMBER_RECIPES                         = 0x1CB0,
    SMSG_GUILD_MEMBER_UPDATE_NOTE                     = 0x7CA0,
    SMSG_GUILD_MOVE_COMPLETE                          = 0x11B2,
    SMSG_GUILD_MOVE_STARTING                          = 0x70A4,
    SMSG_GUILD_NEWS_DELETED                           = 0x74A7,
    SMSG_GUILD_NEWS_UPDATE                            = 0x35A7,
    SMSG_GUILD_PARTY_STATE_RESPONSE                   = 0x50A6,
    SMSG_GUILD_PERMISSIONS_QUERY_RESULTS              = 0x34A3,
    SMSG_GUILD_QUERY_RESPONSE                         = 0x0E06,
    SMSG_GUILD_RANK                                   = 0x30B4,
    SMSG_GUILD_RANKS_UPDATE                           = 0x5DA0,
    SMSG_GUILD_RECIPES                                = 0x10B3,
    SMSG_GUILD_RENAMED                                = 0x74A6,
    SMSG_GUILD_REPUTATION_REACTION_CHANGED            = 0x74B0,
    SMSG_GUILD_REPUTATION_WEEKLY_CAP                  = 0x30B7,
    SMSG_GUILD_RESET                                  = 0x1CB5,
    SMSG_GUILD_REWARDS_LIST                           = 0x1DB0,
    SMSG_GUILD_ROSTER                                 = 0x3DA3,
    SMSG_GUILD_SET_NOTE                               = 0x0000,
    SMSG_GUILD_TRADESKILL_UPDATE                      = 0x0000,
    SMSG_GUILD_UPDATE_ROSTER                          = 0x18B0,
    SMSG_GUILD_XP                                     = 0x3DB0,
    SMSG_GUILD_XP_GAIN                                = 0x14A1,
    SMSG_GUILD_XP_UPDATE                              = 0x0000,
    SMSG_HEALTH_UPDATE                                = 0x4734,
    SMSG_HIGHEST_THREAT_UPDATE                        = 0x4104,
    SMSG_HOTFIX_INFO                                  = 0x19B5,
    SMSG_HOTFIX_NOTIFY                                = 0x55A7,
    SMSG_INITIALIZE_FACTIONS                          = 0x4634,
    SMSG_INITIAL_SPELLS                               = 0x0104,
    SMSG_INIT_CURRENCY                                = 0x15A5,
    SMSG_INIT_WORLD_STATES                            = 0x4C15,
    SMSG_INSPECT                                      = 0x0000,
    SMSG_INSPECT_HONOR_STATS                          = 0x79A5,
    SMSG_INSPECT_RATED_BG_STATS                       = 0x19A5,
    SMSG_INSPECT_RESULTS_UPDATE                       = 0x0C14,
    SMSG_INSPECT_TALENT                               = 0x4014,
    SMSG_INSTANCE_LOCK_WARNING_QUERY                  = 0x4F17,
    SMSG_INSTANCE_RESET                               = 0x6F05,
    SMSG_INSTANCE_RESET_FAILED                        = 0x4725,
    SMSG_INSTANCE_SAVE_CREATED                        = 0x0124,
    SMSG_INVALIDATE_DANCE                             = 0x0E27,
    SMSG_INVALIDATE_PLAYER                            = 0x6325,
    SMSG_INVALID_PROMOTION_CODE                       = 0x6F25,
    SMSG_INVENTORY_CHANGE_FAILURE                     = 0x2236,
    SMSG_ITEM_ADD_PASSIVE                             = 0x7CB4,
    SMSG_ITEM_COOLDOWN                                = 0x4D14,
    SMSG_ITEM_ENCHANT_TIME_UPDATE                     = 0x0F27,
    SMSG_ITEM_EXPIRE_PURCHASE_REFUND                  = 0x1CA0,
    SMSG_ITEM_PUSH_RESULT                             = 0x0E15,
    SMSG_ITEM_REFUND_INFO_RESPONSE                    = 0x15A3,
    SMSG_ITEM_REFUND_RESULT                           = 0x5DB1,
    SMSG_ITEM_REMOVE_PASSIVE                          = 0x39A1,
    SMSG_ITEM_SEND_PASSIVE                            = 0x70B1,
    SMSG_ITEM_TEXT_QUERY_RESPONSE                     = 0x2725,
    SMSG_ITEM_TIME_UPDATE                             = 0x2407,
    SMSG_JOINED_BATTLEGROUND_QUEUE                    = 0x0000,
    SMSG_KICK_REASON                                  = 0x0000,
    SMSG_LEARNED_DANCE_MOVES                          = 0x0E05,
    SMSG_LEARNED_SPELL                                = 0x58A2,
    SMSG_LEVELUP_INFO                                 = 0x0435,
    SMSG_LFG_BOOT_PROPOSAL_UPDATE                     = 0x0F05,
    SMSG_LFG_DISABLED                                 = 0x0815,
    SMSG_LFG_JOIN_RESULT                              = 0x38B6,
    SMSG_LFG_OFFER_CONTINUE                           = 0x6B27,
    SMSG_LFG_OPEN_FROM_GOSSIP                         = 0x0000,
    SMSG_LFG_PARTY_INFO                               = 0x2325,
    SMSG_LFG_PLAYER_INFO                              = 0x4B36,
    SMSG_LFG_PLAYER_REWARD                            = 0x6834,
    SMSG_LFG_PROPOSAL_UPDATE                          = 0x7DA6,
    SMSG_LFG_UPDATE_PLAYER                            = 0x1368,
    SMSG_LFG_UPDATE_PARTY                             = 0x1369,
    SMSG_LFG_QUEUE_STATUS                             = 0x78B4,
    SMSG_LFG_ROLE_CHECK_UPDATE                        = 0x0336,
    SMSG_LFG_ROLE_CHOSEN                              = 0x6A26,
    SMSG_LFG_SLOT_INVALID                             = 0x54B5,
    SMSG_LFG_TELEPORT_DENIED                          = 0x0E14,
    SMSG_LFG_UPDATE_LIST                              = 0x0000,
    SMSG_LFG_UPDATE_SEARCH                            = 0x54A1,
    SMSG_LFG_UPDATE_STATUS                            = 0x31A4,
    SMSG_LFG_UPDATE_STATUS_NONE                       = 0x7CA1,
    SMSG_LF_GUILD_APPLICANT_LIST_UPDATED              = 0x10A4,
    SMSG_LF_GUILD_APPLICATIONS_LIST_CHANGED           = 0x71A5,
    SMSG_LF_GUILD_BROWSE_UPDATED                      = 0x1DA3,
    SMSG_LF_GUILD_COMMAND_RESULT                      = 0x54A6,
    SMSG_LF_GUILD_MEMBERSHIP_LIST_UPDATED             = 0x1CA5,
    SMSG_LF_GUILD_POST_UPDATED                        = 0x35B7,
    SMSG_LF_GUILD_RECRUIT_LIST_UPDATED                = 0x1CB2,
    SMSG_LIST_INVENTORY                               = 0x7CB0,
    SMSG_LOAD_CUF_PROFILES                            = 0x50B1,
    SMSG_LOGIN_SETTIMESPEED                           = 0x4D15,
    SMSG_LOGIN_VERIFY_WORLD                           = 0x2005,
    SMSG_LOGOUT_CANCEL_ACK                            = 0x6514,
    SMSG_LOGOUT_COMPLETE                              = 0x2137,
    SMSG_LOGOUT_RESPONSE                              = 0x0524,
    SMSG_LOG_XPGAIN                                   = 0x4514,
    SMSG_LOOT_ALL_PASSED                              = 0x6237,
    SMSG_LOOT_CLEAR_MONEY                             = 0x2B37,
    SMSG_LOOT_CONTENTS                                = 0x11B1,
    SMSG_LOOT_ITEM_NOTIFY                             = 0x6D15,
    SMSG_LOOT_LIST                                    = 0x6807,
    SMSG_LOOT_MASTER_LIST                             = 0x0325,
    SMSG_LOOT_MONEY_NOTIFY                            = 0x2836,
    SMSG_LOOT_RELEASE_RESPONSE                        = 0x6D25,
    SMSG_LOOT_REMOVED                                 = 0x6817,
    SMSG_LOOT_RESPONSE                                = 0x4C16,
    SMSG_LOOT_ROLL                                    = 0x6507,
    SMSG_LOOT_ROLL_WON                                = 0x6617,
    SMSG_LOOT_SLOT_CHANGED                            = 0x2935,
    SMSG_LOOT_START_ROLL                              = 0x2227,
    SMSG_MAIL_LIST_RESULT                             = 0x4217,
    SMSG_MAP_OBJ_EVENTS                               = 0x54B2,
    SMSG_MEETINGSTONE_COMPLETE                        = 0x2524,
    SMSG_MEETINGSTONE_IN_PROGRESS                     = 0x2D35,
    SMSG_MEETINGSTONE_MEMBER_ADDED                    = 0x0000,
    SMSG_MEETINGSTONE_SETQUEUE                        = 0x0000,
    SMSG_MESSAGECHAT                                  = 0x2026,
    SMSG_MESSAGE_BOX                                  = 0x30A1,
    SMSG_MINIGAME_SETUP                               = 0x6727,
    SMSG_MINIGAME_STATE                               = 0x2E17,
    SMSG_MIRRORIMAGE_DATA                             = 0x2634,
    SMSG_MISSILE_CANCEL                               = 0x3DB4,
    SMSG_MODIFY_COOLDOWN                              = 0x6016,
    SMSG_MONEY_NOTIFY                                 = 0x55B6,
    SMSG_MONSTER_MOVE                                 = 0x6E17,
    SMSG_MONSTER_MOVE_TRANSPORT                       = 0x2004,
    SMSG_MOTD                                         = 0x0A35,
    SMSG_MOUNTRESULT                                  = 0x2225,
    SMSG_MOUNTSPECIAL_ANIM                            = 0x0217,
    SMSG_MOVE_COLLISION_DISABLE                       = 0x31B0,
    SMSG_MOVE_COLLISION_ENABLE                        = 0x11A7,
    SMSG_MOVE_DISABLE_COLLISION                       = 0x0000,
    SMSG_MOVE_DISABLE_GRAVITY                         = 0x0000,
    SMSG_MOVE_ENABLE_COLLISION                        = 0x0000,
    SMSG_MOVE_ENABLE_GRAVITY                          = 0x0000,
    SMSG_MOVE_FEATHER_FALL                            = 0x79B0,
    SMSG_MOVE_GRAVITY_DISABLE                         = 0x75B2,
    SMSG_MOVE_GRAVITY_ENABLE                          = 0x30B3,
    SMSG_MOVE_KNOCK_BACK                              = 0x5CB4,
    SMSG_MOVE_LAND_WALK                               = 0x34B7,
    SMSG_MOVE_NORMAL_FALL                             = 0x51B6,
    SMSG_MOVE_ROOT                                    = 0x7DA0,
    SMSG_MOVE_SET_ACTIVE_MOVER                        = 0x11B3,
    SMSG_MOVE_SET_CAN_FLY                             = 0x3DA1,
    SMSG_MOVE_SET_CAN_FLY_TO_SWIM                     = 0x59A2, // old SMSG_MOVE_SET_CAN_TRANSITION_BETWEEN_SWIM_AND_FLY
    SMSG_MOVE_SET_COLLISION_HEIGHT                    = 0x11B0,
    SMSG_MOVE_SET_COMPOUND_STATE                      = 0x75A0,
    SMSG_MOVE_SET_FLIGHT_BACK_SPEED                   = 0x30A2,
    SMSG_MOVE_SET_FLIGHT_SPEED                        = 0x71A6,
    SMSG_MOVE_SET_HOVER                               = 0x5CB3,
    SMSG_MOVE_SET_PITCH_RATE                          = 0x75B0,
    SMSG_MOVE_SET_RUN_BACK_SPEED                      = 0x71B1,
    SMSG_MOVE_SET_RUN_SPEED                           = 0x3DB5,
    SMSG_MOVE_SET_SWIM_BACK_SPEED                     = 0x5CA6,
    SMSG_MOVE_SET_SWIM_SPEED                          = 0x15A7,
    SMSG_MOVE_SET_TURN_RATE                           = 0x30A5,
    SMSG_MOVE_SET_VEHICLE_REC_ID                      = 0x0000,
    SMSG_MOVE_SET_WALK_IN_AIR                         = 0x0000,
    SMSG_MOVE_SET_WALK_SPEED                          = 0x1DA4,
    SMSG_MOVE_UNROOT                                  = 0x7DB4,
    SMSG_MOVE_UNSET_CAN_FLY                           = 0x15A2,
    SMSG_MOVE_UNSET_CAN_FLY_TO_SWIM                   = 0x7DB2, // old SMSG_MOVE_UNSET_CAN_TRANSITION_BETWEEN_SWIM_AND_FLY
    SMSG_MOVE_UNSET_HOVER                             = 0x51B3,
    SMSG_MOVE_UNSET_WALK_IN_AIR                       = 0x0000,
    SMSG_MOVE_UPDATE_COLLISION_HEIGHT                 = 0x59A3,
    SMSG_MOVE_UPDATE_FLIGHT_BACK_SPEED                = 0x74A0,
    SMSG_MOVE_UPDATE_KNOCK_BACK                       = 0x3DB2,
    SMSG_MOVE_UPDATE_PITCH_RATE                       = 0x1DB5,
    SMSG_MOVE_UPDATE_RUN_BACK_SPEED                   = 0x3DA6,
    SMSG_MOVE_UPDATE_SWIM_BACK_SPEED                  = 0x30B5,
    SMSG_MOVE_UPDATE_SWIM_SPEED                       = 0x59B5,
    SMSG_MOVE_UPDATE_TELEPORT                         = 0x50B2,
    SMSG_MOVE_UPDATE_TURN_RATE                        = 0x5DA1,
    SMSG_MOVE_UPDATE_WALK_SPEED                       = 0x54A2,
    SMSG_MOVE_WATER_WALK                              = 0x75B1,
    SMSG_MULTIPLE_PACKETS                             = 0x6736,
    SMSG_NAME_QUERY_RESPONSE                          = 0x6E04,
    SMSG_NEW_TAXI_PATH                                = 0x4B35,
    SMSG_NEW_WORLD                                    = 0x79B1,
    SMSG_NEW_WORLD_ABORT                              = 0x14B7,
    SMSG_NOTIFICATION                                 = 0x14A0,
    SMSG_NOTIFY_DANCE                                 = 0x4904,
    SMSG_NOTIFY_DEST_LOC_SPELL_CAST                   = 0x6204,
    SMSG_NPC_TEXT_UPDATE                              = 0x4436,
    SMSG_NPC_WONT_TALK                                = 0x0000,
    SMSG_OFFER_PETITION_ERROR                         = 0x2716,
    SMSG_ON_CANCEL_EXPECTED_RIDE_VEHICLE_AURA         = 0x4D34,
    SMSG_OPEN_CONTAINER                               = 0x4714,
    SMSG_OPEN_LFG_DUNGEON_FINDER                      = 0x2C37,
    SMSG_OVERRIDE_LIGHT                               = 0x4225,
    SMSG_PAGE_TEXT_QUERY_RESPONSE                     = 0x2B14,
    SMSG_PARTYKILLLOG                                 = 0x4937,
    SMSG_PARTY_COMMAND_RESULT                         = 0x6E07,
    SMSG_PARTY_MEMBER_STATS                           = 0x2104,
    SMSG_PARTY_MEMBER_STATS_FULL                      = 0x0215,
    SMSG_PAUSE_MIRROR_TIMER                           = 0x4015,
    SMSG_PERIODICAURALOG                              = 0x0416,
    SMSG_PETGODMODE                                   = 0x2E36,
    SMSG_PETITION_ALREADY_SIGNED                      = 0x5DA3,
    SMSG_PETITION_QUERY_RESPONSE                      = 0x4B37,
    SMSG_PETITION_SHOWLIST                            = 0x6405,
    SMSG_PETITION_SHOW_SIGNATURES                     = 0x0716,
    SMSG_PETITION_SIGN_RESULTS                        = 0x6217,
    SMSG_PET_ACTION_FEEDBACK                          = 0x0807,
    SMSG_PET_ACTION_SOUND                             = 0x4324,
    SMSG_PET_ADDED                                    = 0x3CA5,
    SMSG_PET_BROKEN                                   = 0x2E27,
    SMSG_PET_CAST_FAILED                              = 0x2B15,
    SMSG_PET_DISMISS_SOUND                            = 0x2B05,
    SMSG_PET_GUIDS                                    = 0x2D26,
    SMSG_PET_LEARNED_SPELL                            = 0x0507,
    SMSG_PET_MODE                                     = 0x2235,
    SMSG_PET_NAME_INVALID                             = 0x6007,
    SMSG_PET_NAME_QUERY_RESPONSE                      = 0x4C37,
    SMSG_PET_REMOVED_SPELL                            = 0x6A04,
    SMSG_PET_RENAMEABLE                               = 0x2B27,
    SMSG_PET_SLOT_UPDATED                             = 0x51A3,
    SMSG_PET_SPELLS                                   = 0x4114,
    SMSG_PET_TAME_FAILURE                             = 0x6B24,
    SMSG_PET_UPDATE_COMBO_POINTS                      = 0x4325,
    SMSG_PLAYED_TIME                                  = 0x6037,
    SMSG_PLAYERBINDERROR                              = 0x6A24,
    SMSG_PLAYERBOUND                                  = 0x2516,
    SMSG_PLAYER_DIFFICULTY_CHANGE                     = 0x2217,
    SMSG_PLAYER_MOVE                                  = 0x79A2,
    SMSG_PLAYER_SKINNED                               = 0x0116,
    SMSG_PLAYER_UNK_DEAD_ALIVE                        = 0x0000,
    SMSG_PLAYER_VEHICLE_DATA                          = 0x4115,
    SMSG_PLAY_DANCE                                   = 0x4704,
    SMSG_PLAY_MUSIC                                   = 0x4B06,
    SMSG_PLAY_OBJECT_SOUND                            = 0x2635,
    SMSG_PLAY_ONE_SHOT_ANIM_KIT                       = 0x4A35,
    SMSG_PLAY_SOUND                                   = 0x2134,
    SMSG_PLAY_SPELL_VISUAL                            = 0x10B1,
    SMSG_PLAY_SPELL_VISUAL_KIT                        = 0x55A5,
    SMSG_PLAY_TIME_WARNING                            = 0x4814,
    SMSG_PONG                                         = 0x4D42,
    SMSG_POWER_UPDATE                                 = 0x4A07,
    SMSG_PRE_RESURRECT                                = 0x6C36,
    SMSG_PROCRESIST                                   = 0x0426,
    SMSG_PROPOSE_LEVEL_GRANT                          = 0x6114,
    SMSG_PUREMOUNT_CANCELLED_OBSOLETE                 = 0x0000,
    SMSG_PVP_CREDIT                                   = 0x6015,
    SMSG_PVP_LOG_DATA                                 = 0x5CB2,
    SMSG_PVP_OPTIONS_ENABLED                          = 0x50A1,
    SMSG_QUERY_QUESTS_COMPLETED_RESPONSE              = 0x6314,
    SMSG_QUERY_TIME_RESPONSE                          = 0x2124,
    SMSG_QUESTGIVER_OFFER_REWARD                      = 0x2427,
    SMSG_QUESTGIVER_QUEST_COMPLETE                    = 0x55A4,
    SMSG_QUESTGIVER_QUEST_DETAILS                     = 0x2425,
    SMSG_QUESTGIVER_QUEST_FAILED                      = 0x4236,
    SMSG_QUESTGIVER_QUEST_INVALID                     = 0x4016,
    SMSG_QUESTGIVER_QUEST_LIST                        = 0x0134,
    SMSG_QUESTGIVER_REQUEST_ITEMS                     = 0x6236,
    SMSG_QUESTGIVER_STATUS                            = 0x2115,
    SMSG_QUESTGIVER_STATUS_MULTIPLE                   = 0x4F25,
    SMSG_QUESTLOG_FULL                                = 0x0E36,
    SMSG_QUESTUPDATE_ADD_ITEM                         = 0x0000,
    SMSG_QUESTUPDATE_ADD_KILL                         = 0x0D27,
    SMSG_QUESTUPDATE_ADD_PVP_KILL                     = 0x4416,
    SMSG_QUESTUPDATE_COMPLETE                         = 0x2937,
    SMSG_QUESTUPDATE_FAILED                           = 0x6324,
    SMSG_QUESTUPDATE_FAILEDTIMER                      = 0x6427,
    SMSG_QUEST_NPC_QUERY_RESPONSE                     = 0x75A1,
    SMSG_QUEST_CONFIRM_ACCEPT                         = 0x6F07,
    SMSG_QUEST_FORCE_REMOVE                           = 0x6605,
    SMSG_QUEST_POI_QUERY_RESPONSE                     = 0x6304,
    SMSG_QUEST_QUERY_RESPONSE                         = 0x6936,
    SMSG_RAID_GROUP_ONLY                              = 0x0837,
    SMSG_RAID_INSTANCE_INFO                           = 0x6626,
    SMSG_RAID_INSTANCE_MESSAGE                        = 0x6E15,
    SMSG_RAID_MARKERS_CHANGED                         = 0x10A1,
    SMSG_RAID_READY_CHECK_THROTTLED_ERROR             = 0x2607,
    SMSG_RAID_SUMMON_FAILED                           = 0x18B6,
    SMSG_RANDOMIZE_CHAR_NAME                          = 0x38B1,
    SMSG_RATED_BG_RATING                              = 0x15A1,
    SMSG_RATED_BG_STATS                               = 0x34A1,
    SMSG_READ_ITEM_FAILED                             = 0x0F16,
    SMSG_READ_ITEM_OK                                 = 0x2605,
    SMSG_REALM_SPLIT                                  = 0x2714,
    SMSG_REAL_GROUP_UPDATE                            = 0x0F34,
    SMSG_RECEIVED_MAIL                                = 0x2924,
    SMSG_REDIRECT_CLIENT                              = 0x0942,
    SMSG_REFER_A_FRIEND_EXPIRED                       = 0x4934,
    SMSG_REFER_A_FRIEND_FAILURE                       = 0x2037,
    SMSG_REFORGE_RESULT                               = 0x58A4,
    SMSG_REMOVED_SPELL                                = 0x4804,
    SMSG_REPORT_PVP_AFK_RESULT                        = 0x2D06,
    SMSG_REQUEST_CEMETERY_LIST_RESPONSE               = 0x30A7,
    SMSG_REQUEST_PVP_REWARDS_RESPONSE                 = 0x5DA4,
    SMSG_RESEARCH_COMPLETE                            = 0x35A6,
    SMSG_RESEARCH_SETUP_HISTORY                       = 0x10B6,
    SMSG_RESET_COMPRESSION_CONTEXT                    = 0x0142,
    SMSG_RESET_FAILED_NOTIFY                          = 0x4616,
    SMSG_RESISTLOG                                    = 0x0000,
    SMSG_RESPOND_INSPECT_ACHIEVEMENTS                 = 0x15B0,
    SMSG_RESURRECT_REQUEST                            = 0x2905,
    SMSG_RESYNC_RUNES                                 = 0x6224,
    SMSG_ROLE_POLL_BEGIN                              = 0x70B0,
    SMSG_RWHOIS                                       = 0x2437,
    SMSG_SELL_ITEM                                    = 0x6105,
    SMSG_SEND_MAIL_RESULT                             = 0x4927,
    SMSG_SEND_UNLEARN_SPELLS                          = 0x4E25,
    SMSG_SERVERTIME                                   = 0x6327,
    SMSG_SERVER_FIRST_ACHIEVEMENT                     = 0x6424,
    SMSG_SERVER_INFO_RESPONSE                         = 0x74B5,
    SMSG_SERVER_MESSAGE                               = 0x6C04,
    SMSG_SERVER_PERF                                  = 0x74B6,
    SMSG_SET_AI_ANIM_KIT                              = 0x0000,
    SMSG_SET_DF_FAST_LAUNCH_RESULT                    = 0x35B6,
    SMSG_SET_FACTION_ATWAR                            = 0x4216,
    SMSG_SET_FACTION_STANDING                         = 0x0126,
    SMSG_SET_FACTION_VISIBLE                          = 0x2525,
    SMSG_SET_FLAT_SPELL_MODIFIER                      = 0x2834,
    SMSG_SET_FORCED_REACTIONS                         = 0x4615,
    SMSG_SET_MELEE_ANIM_KIT                           = 0x0000,
    SMSG_SET_MOVEMENT_ANIM_KIT                        = 0x0000,
    SMSG_SET_PCT_SPELL_MODIFIER                       = 0x0224,
    SMSG_SET_PHASE_SHIFT                              = 0x70A0,
    SMSG_SET_PLAYER_DECLINED_NAMES_RESULT             = 0x2B25,
    SMSG_SET_PLAY_HOVER_ANIM                          = 0x30A6,
    SMSG_SET_PROFICIENCY                              = 0x6207,
    SMSG_SET_PROJECTILE_POSITION                      = 0x2616,
    SMSG_SHOWTAXINODES                                = 0x2A36,
    SMSG_SHOW_BANK                                    = 0x2627,
    SMSG_SHOW_RATINGS                                 = 0x11B4,
    SMSG_SOCKET_GEMS_RESULT                           = 0x6014,
    SMSG_SOR_START_EXPERIENCE_INCOMPLETE              = 0x7CA7,
    SMSG_SPELLBREAKLOG                                = 0x6B17,
    SMSG_SPELLDAMAGESHIELD                            = 0x2927,
    SMSG_SPELLDISPELLOG                               = 0x4516,
    SMSG_SPELLENERGIZELOG                             = 0x0414,
    SMSG_SPELLHEALLOG                                 = 0x2816,
    SMSG_SPELLINSTAKILLLOG                            = 0x6216,
    SMSG_SPELLINTERRUPTLOG                            = 0x1DA7,
    SMSG_SPELLLOGEXECUTE                              = 0x0626,
    SMSG_SPELLLOGMISS                                 = 0x0625,
    SMSG_SPELLNONMELEEDAMAGELOG                       = 0x4315,
    SMSG_SPELLORDAMAGE_IMMUNE                         = 0x4507,
    SMSG_SPELLSTEALLOG                                = 0x4E26,
    SMSG_SPELL_CATEGORY_COOLDOWN                      = 0x71B6,
    SMSG_SPELL_COOLDOWN                               = 0x4B16,
    SMSG_SPELL_DELAYED                                = 0x0715,
    SMSG_SPELL_FAILED_OTHER                           = 0x0C34,
    SMSG_SPELL_FAILURE                                = 0x4535,
    SMSG_SPELL_GO                                     = 0x6E16,
    SMSG_SPELL_START                                  = 0x6415,
    SMSG_SPELL_UPDATE_CHAIN_TARGETS                   = 0x6006,
    SMSG_SPIRIT_HEALER_CONFIRM                        = 0x4917,
    SMSG_SPLINE_MOVE_COLLISION_DISABLE                = 0x35B1,
    SMSG_SPLINE_MOVE_COLLISION_ENABLE                 = 0x3CB0,
    SMSG_SPLINE_MOVE_GRAVITY_DISABLE                  = 0x5DB5,
    SMSG_SPLINE_MOVE_GRAVITY_ENABLE                   = 0x3CA6,
    SMSG_SPLINE_MOVE_ROOT                             = 0x51B4,
    SMSG_SPLINE_MOVE_SET_ANIM                         = 0x4335,
    SMSG_SPLINE_MOVE_SET_FEATHER_FALL                 = 0x3DA5,
    SMSG_SPLINE_MOVE_SET_FLIGHT_BACK_SPEED            = 0x38B3,
    SMSG_SPLINE_MOVE_SET_FLIGHT_SPEED                 = 0x39A0,
    SMSG_SPLINE_MOVE_SET_FLYING                       = 0x31B5,
    SMSG_SPLINE_MOVE_SET_HOVER                        = 0x14B6,
    SMSG_SPLINE_MOVE_SET_LAND_WALK                    = 0x3DA7,
    SMSG_SPLINE_MOVE_SET_NORMAL_FALL                  = 0x38B2,
    SMSG_SPLINE_MOVE_SET_PITCH_RATE                   = 0x14B0,
    SMSG_SPLINE_MOVE_SET_RUN_BACK_SPEED               = 0x3DB3,
    SMSG_SPLINE_MOVE_SET_RUN_MODE                     = 0x75A7,
    SMSG_SPLINE_MOVE_SET_RUN_SPEED                    = 0x51B7,
    SMSG_SPLINE_MOVE_SET_SWIM_BACK_SPEED              = 0x59A1,
    SMSG_SPLINE_MOVE_SET_SWIM_SPEED                   = 0x39A4,
    SMSG_SPLINE_MOVE_SET_TURN_RATE                    = 0x78B5,
    SMSG_SPLINE_MOVE_SET_WALK_MODE                    = 0x54B6,
    SMSG_SPLINE_MOVE_SET_WALK_SPEED                   = 0x34A5,
    SMSG_SPLINE_MOVE_START_SWIM                       = 0x31A5,
    SMSG_SPLINE_MOVE_STOP_SWIM                        = 0x1DA2,
    SMSG_SPLINE_MOVE_UNROOT                           = 0x75B6,
    SMSG_SPLINE_MOVE_UNSET_FLYING                     = 0x58A6,
    SMSG_SPLINE_MOVE_UNSET_HOVER                      = 0x7DA5,
    SMSG_SPLINE_MOVE_SET_WATER_WALK                   = 0x50A2,
    SMSG_STABLE_RESULT                                = 0x2204,
    SMSG_STANDSTATE_UPDATE                            = 0x6F04,
    SMSG_START_MIRROR_TIMER                           = 0x6824,
    SMSG_START_TIMER                                  = 0x59A5,
    SMSG_STOP_DANCE                                   = 0x4637,
    SMSG_STOP_MIRROR_TIMER                            = 0x0B06,
    SMSG_STREAMING_MOVIE                              = 0x15B7,
    SMSG_SUMMON_CANCEL                                = 0x0B34,
    SMSG_SUMMON_REQUEST                               = 0x2A07,
    SMSG_SUPERCEDED_SPELL                             = 0x35B0,
    SMSG_SUPPRESS_NPC_GREETINGS                       = 0x74B1,
    SMSG_SUSPEND_COMMS                                = 0x4140,
    SMSG_SUSPEND_TOKEN_RESPONSE                       = 0x14B1,
    SMSG_TALENTS_ERROR                                = 0x0916,
    SMSG_TALENTS_INFO                                 = 0x6F26,
    SMSG_TALENTS_INVOLUNTARILY_RESET                  = 0x2C27,
    SMSG_TAXINODE_STATUS                              = 0x2936,
    SMSG_TEST_DROP_RATE_RESULT                        = 0x6816,
    SMSG_TEXT_EMOTE                                   = 0x0B05,
    SMSG_THREAT_CLEAR                                 = 0x6437,
    SMSG_THREAT_REMOVE                                = 0x2E05,
    SMSG_THREAT_UPDATE                                = 0x4735,
    SMSG_TIME_ADJUSTMENT                              = 0x79B7,
    SMSG_TIME_SYNC_REQ                                = 0x3CA4,
    SMSG_TITLE_EARNED                                 = 0x2426,
    SMSG_TOGGLE_XP_GAIN                               = 0x6704,
    SMSG_TOTEM_CREATED                                = 0x2414,
    SMSG_TRADE_STATUS                                 = 0x5CA3,
    SMSG_TRADE_STATUS_EXTENDED                        = 0x70A2,
    SMSG_TRAINER_BUY_FAILED                           = 0x0004,
    SMSG_TRAINER_BUY_SUCCEEDED                        = 0x6A05,
    SMSG_TRAINER_LIST                                 = 0x4414,
    SMSG_TRANSFER_ABORTED                             = 0x0537,
    SMSG_TRANSFER_PENDING                             = 0x18A6,
    SMSG_TRIGGER_CINEMATIC                            = 0x6C27,
    SMSG_TRIGGER_MOVIE                                = 0x4625,
    SMSG_TURN_IN_PETITION_RESULTS                     = 0x0F07,
    SMSG_TUTORIAL_FLAGS                               = 0x0B35,
    SMSG_UNIT_HEALTH_FREQUENT                         = 0x2C26,
    SMSG_UNIT_SPELLCAST_START                         = 0x2517,
    SMSG_UPDATE_ACCOUNT_DATA                          = 0x6837,
    SMSG_UPDATE_ACCOUNT_DATA_COMPLETE                 = 0x2015,
    SMSG_UPDATE_COMBO_POINTS                          = 0x6B34,
    SMSG_UPDATE_CURRENCY                              = 0x59B0,
    SMSG_UPDATE_CURRENCY_WEEK_LIMIT                   = 0x70A7,
    SMSG_UPDATE_DUNGEON_ENCOUNTER_FOR_LOOT            = 0x3CB5,
    SMSG_UPDATE_INSTANCE_ENCOUNTER_UNIT               = 0x4007,
    SMSG_UPDATE_INSTANCE_OWNERSHIP                    = 0x4915,
    SMSG_UPDATE_LAST_INSTANCE                         = 0x0437,
    SMSG_UPDATE_OBJECT                                = 0x4715,
    SMSG_UPDATE_SERVER_PLAYER_POSITION                = 0x74A3,
    SMSG_UPDATE_WORLD_STATE                           = 0x4816,
    SMSG_USERLIST_ADD                                 = 0x0F37,
    SMSG_USERLIST_REMOVE                              = 0x2006,
    SMSG_USERLIST_UPDATE                              = 0x0135,
    SMSG_VOICESESSION_FULL                            = 0x6225,
    SMSG_VOICE_CHAT_STATUS                            = 0x0F15,
    SMSG_VOICE_PARENTAL_CONTROLS                      = 0x0534,
    SMSG_VOICE_SESSION_LEAVE                          = 0x2A24,
    SMSG_VOICE_SESSION_ROSTER_UPDATE                  = 0x2A17,
    SMSG_VOICE_SET_TALKER_MUTED                       = 0x6E35,
    SMSG_VOID_ITEM_SWAP_RESPONSE                      = 0x78A2,
    SMSG_VOID_STORAGE_CONTENTS                        = 0x75B4,
    SMSG_VOID_STORAGE_FAILED                          = 0x18A7,
    SMSG_VOID_STORAGE_TRANSFER_CHANGES                = 0x51A6,
    SMSG_VOID_TRANSFER_RESULT                         = 0x1DA6,
    SMSG_WAIT_QUEUE_FINISH                            = 0x75B7,
    SMSG_WAIT_QUEUE_UPDATE                            = 0x58A1,
    SMSG_WARDEN_DATA                                  = 0x31A0,
    SMSG_WARGAME_CHECK_ENTRY                          = 0x3DA4,
    SMSG_WARGAME_REQUEST_SENT                         = 0x59B2,
    SMSG_WEATHER                                      = 0x2904,
    SMSG_WEEKLY_LAST_RESET                            = 0x50A5,
    SMSG_WEEKLY_RESET_CURRENCY                        = 0x3CA1,
    SMSG_WEEKLY_SPELL_USAGE                           = 0x39B7,
    SMSG_WEEKLY_SPELL_USAGE_UPDATE                    = 0x11B5,
    SMSG_WHO                                          = 0x6907,
    SMSG_WHOIS                                        = 0x6917,
    SMSG_WORLD_SERVER_INFO                            = 0x31A2,
    SMSG_WORLD_STATE_UI_TIMER_UPDATE                  = 0x4A14,
    SMSG_XP_GAIN_ABORTED                              = 0x50B4,
    SMSG_ZONE_UNDER_ATTACK                            = 0x0A06,
};

/// Player state
enum SessionStatus
{
    STATUS_AUTHED = 0,                                      // Player authenticated (_player == NULL, m_playerRecentlyLogout = false or will be reset before handler call, m_GUID have garbage)
    STATUS_LOGGEDIN,                                        // Player in game (_player != NULL, m_GUID == _player->GetGUID(), inWorld())
    STATUS_TRANSFER,                                        // Player transferring to another map (_player != NULL, m_GUID == _player->GetGUID(), !inWorld())
    STATUS_LOGGEDIN_OR_RECENTLY_LOGGOUT,                    // _player != NULL or _player == NULL && m_playerRecentlyLogout && m_playerLogout, m_GUID store last _player guid)
    STATUS_NEVER,                                           // Opcode not accepted from client (deprecated or server side only)
    STATUS_UNHANDLED                                        // Opcode not handled yet
};

enum PacketProcessing
{
    PROCESS_INPLACE = 0,                                    //process packet whenever we receive it - mostly for non-handled or non-implemented packets
    PROCESS_THREADUNSAFE,                                   //packet is not thread-safe - process it in World::UpdateSessions()
    PROCESS_THREADSAFE                                      //packet is thread-safe - process it in Map::Update()
};

class WorldPacket;
class WorldSession;

typedef void(WorldSession::*pOpcodeHandler)(WorldPacket& recvPacket);

struct OpcodeHandler
{
    OpcodeHandler() {}
    OpcodeHandler(char const* _name, SessionStatus _status, PacketProcessing _processing, pOpcodeHandler _handler)
        : Name(_name), Status(_status), ProcessingPlace(_processing), Handler(_handler) {}

    char const* Name;
    SessionStatus Status;
    PacketProcessing ProcessingPlace;
    pOpcodeHandler Handler;
};

class OpcodeTable
{
    public:
        OpcodeTable()
        {
            memset(_internalTable, 0, sizeof(_internalTable));
        }

        ~OpcodeTable()
        {
            for (uint16 i = 0; i < NUM_OPCODE_HANDLERS; ++i)
                delete _internalTable[i];
        }

        void Initialize();

        OpcodeHandler const* operator[](uint32 index) const
        {
            return _internalTable[index];
        }

    private:
        template<bool isInValidRange, bool isNonZero>
        void ValidateAndSetOpcode(uint16 opcode, char const* name, SessionStatus status, PacketProcessing processing, pOpcodeHandler handler);

        // Prevent copying this structure
        OpcodeTable(OpcodeTable const&);
        OpcodeTable& operator=(OpcodeTable const&);

        OpcodeHandler* _internalTable[NUM_OPCODE_HANDLERS];
};

extern OpcodeTable opcodeTable;
extern uint32 opcodePerSecond[NUM_OPCODE_HANDLERS];

void InitOpcodes();

/// Lookup opcode name for human understandable logging
inline std::string GetOpcodeNameForLogging(Opcodes id)
{
    uint32 opcode = uint32(id);
    std::ostringstream ss;
    ss << '[';

    if (id < UNKNOWN_OPCODE)
    {
        if (OpcodeHandler const* handler = opcodeTable[uint32(id) & 0x7FFF])
        {
            ss << handler->Name;
            if (opcode & COMPRESSED_OPCODE_MASK)
                ss << "_COMPRESSED";
        }
        else
            ss << "UNKNOWN OPCODE";
    }
    else
        ss << "INVALID OPCODE";

    ss << " 0x" << std::hex << std::uppercase << opcode << std::nouppercase << " (" << std::dec << opcode << ")]";
    return ss.str();
}

#endif
/// @}
